#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using dnp.DomainModeler;
using dnp.DomainModeler.Configuration;
using MbUnit.Framework;

namespace dnp.DomainModeler.Test
{
    [TestFixture]
    public class DomainTest
    {
        private Domain _domain = null;

        [TestFixtureSetUp]
        public void Setup()
        {
            DomainModelDefinition def = dnp.Framework.Reflection.Serializer.DeserializeFromResource<DomainModelDefinition>("ReferenceSchema-DomainDefinition.xml");
            _domain = new Domain(def);
        }

        [Test]
        public void ResolveLooseRelationships()
        {
            
        }

        //[Test]
        //public void DomainNamespace()
        //{
        //    Assert.AreEqual("dotNetProfessional.ReferenceSchema.Business.Entities", _domain.DomainNamespace);
        //}

        [Test]
        public void ClientName()
        {
            Assert.AreEqual("dotNetProfessional", _domain.ClientName);
        }

        [Test]
        public void ModelName()
        {
            Assert.AreEqual("ReferenceSchema", _domain.ModelName);
        }

        [Test]
        public void EntitiesByDependancy_InsertOrder()
        {
            EntityCollection entities = _domain.EntitiesByDependancy(EntityOrderEnum.InsertOrder);

            string[] expected = {
                                    "AllDataTypes", "Country", "Description", "Home", "SingleTable", "State", "University", "Course", "Person",
                                    "Region", "Dealer", "Car", "Student", "PreviousUniversityStudent"
                                };

            Assert.AreEqual(expected.Length, entities.Count);

            for (int i = 0; i < entities.Count; i++)
            {
                Assert.AreEqual(expected[i], entities[i].Name);
            }

        }

        [Test]
        public void EntitiesByDependancy_DeleteOrder()
        {
            EntityCollection entities = _domain.EntitiesByDependancy(EntityOrderEnum.DeleteOrder);

            string[] expected = {
                                    "PreviousUniversityStudent", "Student", "Car", "Dealer", "Region", "Person", "Course", "University", "State", "SingleTable", "Home",
                                    "Description", "Country", "AllDataTypes"
                                };

            Assert.AreEqual(expected.Length, entities.Count);

            for (int i = 0; i < entities.Count; i++)
            {
                Assert.AreEqual(expected[i], entities[i].Name);
            }

        }
    }
}
