#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections.Generic;
using System.Text;
using dnp.DomainModeler;
using dnp.DomainModeler.Configuration;
using MbUnit.Framework;

namespace dnp.DomainModeler.Test
{
    [TestFixture]
    public class PropertyTest
    {
        private Domain _domain = null;

        [TestFixtureSetUp]
        public void Setup()
        {
            DomainModelDefinition def = dnp.Framework.Reflection.Serializer.DeserializeFromResource<DomainModelDefinition>("ReferenceSchema-DomainDefinition.xml");
            _domain = new Domain(def);
        }

        [Test]
        public void Name()
        {
            // Table
            Entity entity = _domain.Entities["Car"];
            Assert.AreEqual(6, entity.Properties().Count); // Visible properties
            Assert.AreEqual("Make", entity.Properties()[0].Name);
            Assert.AreEqual("Model", entity.Properties()[1].Name);
            Assert.AreEqual("AverageFuelEconomy", entity.Properties()[5].Name);

            // View
            entity = _domain.Entities["CourseStudent"];
            Assert.AreEqual(4, entity.Properties().Count);
            Assert.AreEqual("Name", entity.Properties()[0].Name);
            Assert.AreEqual("CourseId", entity.Properties()[1].Name); // This is a view and doesnt have 'relationships' so its a property
            Assert.AreEqual("FirstName", entity.Properties()[2].Name);
            Assert.AreEqual("LastName", entity.Properties()[3].Name);
        }

        [Test]
        public void Source()
        {
            Entity entity = _domain.Entities["Car"];
            Assert.AreEqual(6, entity.Properties().Count); // Visibile properties
            Assert.AreEqual("Make", entity.Properties()[0].Source.Name);
            Assert.AreEqual("Model", entity.Properties()[1].Source.Name);
            Assert.AreEqual("AverageFuelEconomy", entity.Properties()[5].Source.Name);
        }

        [Test]
        public void Domain()
        {
            Entity entity = _domain.Entities["Car"];
            Assert.IsNotNull(entity.Domain);
        }

        [Test]
        public void Visible()
        {
            Entity entity = _domain.Entities["AllDataTypes"];
            Assert.AreEqual(true, entity.Properties()["Char"].Visible);
            Assert.AreEqual(true, entity.Properties(VisibilityEnum.Visible)["Char"].Visible);
            Assert.AreEqual(false, entity.Properties(VisibilityEnum.Hidden)["Text"].Visible);
            Assert.AreEqual(true, entity.Properties(VisibilityEnum.All)["Int"].Visible);
            
            // These fields shouldn't exist
            Assert.IsFalse(entity.Properties().Contains("Text"));
        }

        [Test]
        public void Parent()
        {
            
        }

        [Test]
        public void InRelationship()
        {
            Assert.AreEqual(true, _domain.Entities["Region"].Properties(PropertyFilterEnum.All, VisibilityEnum.All)["StateId"].InRelationship);
            Assert.AreEqual(true, _domain.Entities["Region"].Properties(PropertyFilterEnum.All, VisibilityEnum.All)["CountryId"].InRelationship);
            Assert.AreEqual(false, _domain.Entities["Region"].Properties(PropertyFilterEnum.All, VisibilityEnum.All)["RegionName"].InRelationship);
        }

        [Test]
        public void Mapped_Person()
        {
            PropertyCollection properties = _domain.Entities["Person"].Properties(PropertyFilterEnum.All, VisibilityEnum.All);
            Assert.AreEqual(false, properties["ChristianName"].IsMapped);
            Assert.AreEqual(false, properties["LastName"].IsMapped);
            Assert.AreEqual(true, properties["HomeId"].IsMapped);
            Assert.AreEqual(true, properties["MotherId"].IsMapped);

            Assert.AreEqual("HomeAddress", properties["HomeId"].Mapped.Name);
            Assert.AreEqual(1, properties["HomeId"].Mapped.Properties.Count);
            Assert.AreEqual("Address", properties["HomeId"].Mapped.Properties[0].Name);

            Assert.AreEqual("Mother", properties["MotherId"].Mapped.Name);
            Assert.AreEqual(2, properties["MotherId"].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", properties["MotherId"].Mapped.Properties[0].Name);
            Assert.AreEqual("LastName", properties["MotherId"].Mapped.Properties[1].Name);

            // Non-Mapped field
            Assert.AreEqual("ChristianName", properties["ChristianName"].Mapped.Name);
            Assert.AreEqual(1, properties["ChristianName"].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", properties["ChristianName"].Mapped.Properties[0].Name);
        }

        [Test]
        public void Mapped_Car()
        {
            PropertyCollection properties = _domain.Entities["Car"].Properties(PropertyFilterEnum.All, VisibilityEnum.All);
            Assert.AreEqual(false, properties["CarId"].IsMapped);
            Assert.AreEqual(false, properties["Make"].IsMapped);
            Assert.AreEqual(false, properties["Model"].IsMapped);
            Assert.AreEqual(true, properties["OwnerId"].IsMapped);
            Assert.AreEqual(true, properties["PreviousOwnerId"].IsMapped);
        }

        #region GetEntityView - QueryView
        [Test]
        public void GetEntityView_Car_Query_EntityLevel()
        {
            Entity e = _domain.Entities["Car"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.Entity);
            int keyIndex = 0;
            Assert.AreEqual(13, propertyView.Count, "Incorrect number of properties.");
            // Car property
            Assert.AreEqual("CarId", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("CarId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // Make property
            Assert.AreEqual("Make", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("Make", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // Model property
            Assert.AreEqual("Model", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("Model", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // OwnerId property (Mapped)
            Assert.AreEqual("OwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(2, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // PreviousOwnerId property (Mapped)
            Assert.AreEqual("PreviousOwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(2, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsTrue(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // RegistrationNumber property
            Assert.AreEqual("RegistrationNumber", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("RegistrationNumber", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // MinFuelEconomy property
            Assert.AreEqual("MinFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("MinFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // MaxFuelEconomy property
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("MaxFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual("MaxFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // AverageFuelEconomy property
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("AverageFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual("AverageFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // DealerName property
            Assert.AreEqual("DealerName", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);

            keyIndex++;
            // DealerStateId property
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("DealerStateId", propertyView[keyIndex].Name);
            Assert.AreEqual("DealerStateId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // OwnerId property
            Assert.AreEqual("OwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("OwnerId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // PreviousOwnerId property
            Assert.AreEqual("PreviousOwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual("PreviousOwnerId", propertyView[keyIndex].Mapped.Properties[0].Name);

            // Just a sanity check that we've check all properties
            Assert.AreEqual(keyIndex, propertyView.Count - 1);

        }

        [Test]
        public void GetEntityView_Student_Query_EntityLevel()
        {
            Entity e = _domain.Entities["Student"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.Entity);

            int keyIndex = 0;
            Assert.AreEqual(2, propertyView.Count, "Incorrect number of properties.");
            // StudentId property
            Assert.AreEqual("StudentId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("StudentId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // Make property
            Assert.AreEqual("GPA", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("GPA", propertyView[keyIndex].Mapped.Properties[0].Name);

            // Just a sanity check that we've check all properties
            Assert.AreEqual(keyIndex, propertyView.Count - 1);

        }

        [Test]
        public void GetEntityView_Student_Query_IncludeBaseClass()
        {
            Entity e = _domain.Entities["Student"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.IncludeBaseClass);

            int keyIndex = 0;
            Assert.AreEqual(9, propertyView.Count, "Incorrect number of properties.");
            // StudentId property
            Assert.AreEqual("StudentId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("StudentId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // Make property
            Assert.AreEqual("GPA", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("GPA", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // PersonID from Person base class (base class primary key)
            Assert.AreEqual("PersonId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("PersonId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // FirstName property its been aliased
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // LastName property
            Assert.AreEqual("LastName", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("LastName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // HomeId property
            Assert.AreEqual("HomeId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("Address", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // FriendId property
            Assert.AreEqual("MotherId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(2, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual("LastName", propertyView[keyIndex].Mapped.Properties[1].Name);

            keyIndex++;
            // HomeId property
            Assert.AreEqual("HomeId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("HomeId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // FriendId property
            Assert.AreEqual("MotherId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("MotherId", propertyView[keyIndex].Mapped.Properties[0].Name);

            // Just a sanity check that we've check all properties
            Assert.AreEqual(keyIndex, propertyView.Count - 1);

        }

        [Test]
        public void GetEntityView_Region_Query_IncludeBaseClass()
        {
            Entity e = _domain.Entities["Region"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.IncludeBaseClass);

            Assert.AreEqual(5, propertyView.Count, "Incorrect number of properties.");

            int keyIndex = 0;
            // StateID property its been aliased
            Assert.AreEqual("StateId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("StateName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // Country property
            Assert.AreEqual("CountryId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("CountryName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // RegionName from Person base class (base class primary key)
            Assert.AreEqual("RegionName", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("RegionName", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // State property
            Assert.AreEqual("StateId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("StateId", propertyView[keyIndex].Mapped.Properties[0].Name);

            keyIndex++;
            // LastName property
            Assert.AreEqual("CountryId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("CountryId", propertyView[keyIndex].Mapped.Properties[0].Name);

            // Just a sanity check that we've check all properties
            Assert.AreEqual(keyIndex, propertyView.Count - 1);

        }

        [Test]
        public void GetEntityView_SingleTable_Query_EntityLevel()
        {
            Entity e = _domain.Entities["SingleTable"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.Entity);

            Assert.AreEqual(7, propertyView.Count, "Incorrect number of properties.");

            int keyIndex = 0;
            // SingleTableId property
            Assert.AreEqual("SingleTableId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("SingleTableId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.PrimaryKey, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // Name property
            Assert.AreEqual("Name", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("Name", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.Field, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // CreatedBy property
            Assert.AreEqual("CreatedBy", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("CreatedBy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.CreatedBy, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ModifiedBy property
            Assert.AreEqual("ModifiedBy", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ModifiedBy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.ModifiedBy, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // CreatedDate property
            Assert.AreEqual("CreatedDate", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("CreatedDate", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.CreatedDate, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ModifiedDate property
            Assert.AreEqual("ModifiedDate", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ModifiedDate", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.ModifiedDate, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ConcurrencyId property
            Assert.AreEqual("ConcurrencyId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ConcurrencyId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.AreEqual(AttributeRoleEnum.ConcurrencyKey, propertyView[keyIndex].AttributeRole);
        }

        [Test]
        public void GetEntityView_Student_Query_DerivedClass()
        {
            Entity e = _domain.Entities["Student"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Query, PropertyDepthFilterEnum.DerivedClass);

            Assert.AreEqual(11, propertyView.Count, "Incorrect number of properties.");
        }
        #endregion

        #region GetEntityView - ClassView
 
        [Test]
        public void GetEntityView_Car_Entity_EntityLevel()
        {
            Entity e = _domain.Entities["Car"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Entity, PropertyDepthFilterEnum.Entity);
            int keyIndex = 0;

            Assert.AreEqual(11, propertyView.Count, "Incorrect number of properties.");
            // Car property
            Assert.AreEqual("CarId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("CarId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // Make property
            Assert.AreEqual("Make", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("Make", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // Model property
            Assert.AreEqual("Model", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("Model", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // OwnerID property
            Assert.AreEqual("OwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(2, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsTrue(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // PreviousOwnerID property
            Assert.AreEqual("PreviousOwnerId", propertyView[keyIndex].Name);
            Assert.AreEqual(true, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(2, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsTrue(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // RegistrationNumber property
            Assert.AreEqual("RegistrationNumber", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("RegistrationNumber", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // MinFuelEconomy property
            Assert.AreEqual("MinFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("MinFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // MaxFuelEconomy property
            Assert.AreEqual("MaxFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("MaxFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // AverageFuelEconomy property
            Assert.AreEqual("AverageFuelEconomy", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("AverageFuelEconomy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // DealerName property
            Assert.AreEqual("DealerName", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("DealerName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // DealerStateID property
            Assert.AreEqual("DealerStateId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("DealerStateId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

        }

        [Test]
        public void GetEntityView_Student_Entity_EntityLevel()
        {
            Entity e = _domain.Entities["Student"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Entity, PropertyDepthFilterEnum.Entity);

            int keyIndex = 0;
            Assert.AreEqual(2, propertyView.Count, "Incorrect number of properties.");
            // StudentID property
            Assert.AreEqual("StudentId", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("StudentId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // Make property
            Assert.AreEqual("GPA", propertyView[keyIndex].Name);
            Assert.AreEqual(false, propertyView[keyIndex].IsMapped);
            Assert.AreEqual(1, propertyView[keyIndex].Mapped.Properties.Count);
            Assert.AreEqual("GPA", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

        }

        [Test]
        public void GetEntityView_SingleTable_Entity_EntityLevel()
        {
            Entity e = _domain.Entities["SingleTable"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Entity, PropertyDepthFilterEnum.Entity);

            Assert.AreEqual(7, propertyView.Count, "Incorrect number of properties.");

            int keyIndex = 0;
            // SingleTableID property
            Assert.AreEqual("SingleTableId", propertyView[keyIndex].Name);
            Assert.AreEqual("SingleTableId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.PrimaryKey, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // Name property
            Assert.AreEqual("Name", propertyView[keyIndex].Name);
            Assert.AreEqual("Name", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.Field, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // CreatedBy property
            Assert.AreEqual("CreatedBy", propertyView[keyIndex].Name);
            Assert.AreEqual("CreatedBy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.CreatedBy, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ModifiedBy property
            Assert.AreEqual("ModifiedBy", propertyView[keyIndex].Name);
            Assert.AreEqual("ModifiedBy", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.ModifiedBy, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // CreatedDate property
            Assert.AreEqual("CreatedDate", propertyView[keyIndex].Name);
            Assert.AreEqual("CreatedDate", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.CreatedDate, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ModifiedDate property
            Assert.AreEqual("ModifiedDate", propertyView[keyIndex].Name);
            Assert.AreEqual("ModifiedDate", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.ModifiedDate, propertyView[keyIndex].AttributeRole);

            keyIndex++;
            // ConcurrencyID property
            Assert.AreEqual("ConcurrencyId", propertyView[keyIndex].Name);
            Assert.AreEqual("ConcurrencyId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);
            Assert.AreEqual(AttributeRoleEnum.ConcurrencyKey, propertyView[keyIndex].AttributeRole);
        }

        [Test]
        public void GetEntityView_Student_Entity_IncludeBaseClass()
        {
            Entity e = _domain.Entities["Student"];
            ViewPropertyCollection propertyView = e.Properties(PropertyViewEnum.Entity, PropertyDepthFilterEnum.IncludeBaseClass);

            Assert.AreEqual(6, propertyView.Count, "Incorrect number of properties.");

            int keyIndex = 0;
            // StudentID property
            Assert.AreEqual("StudentId", propertyView[keyIndex].Name);
            Assert.AreEqual("StudentId", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // Make property
            Assert.AreEqual("GPA", propertyView[keyIndex].Name);
            Assert.AreEqual("GPA", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // FirstName property its been aliased
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Name);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // LastName property
            Assert.AreEqual("LastName", propertyView[keyIndex].Name);
            Assert.AreEqual("LastName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsFalse(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // HomeID property
            Assert.AreEqual("HomeId", propertyView[keyIndex].Name);
            Assert.AreEqual("Address", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsTrue(propertyView[keyIndex].IsMapped);

            keyIndex++;
            // FriendID property
            Assert.AreEqual("MotherId", propertyView[keyIndex].Name);
            Assert.AreEqual("ChristianName", propertyView[keyIndex].Mapped.Properties[0].Name);
            Assert.IsTrue(propertyView[keyIndex].IsMapped);
        }
        #endregion

        [Test]
        public void BusinessKey()
        {
            Entity entity = _domain.Entities["Student"];

            Assert.AreEqual(2, entity.BusinessKey.Count);
            Assert.AreEqual("ChristianName", entity.BusinessKey[0].Name);
            Assert.AreEqual("LastName", entity.BusinessKey[01].Name);
        }

    }
}
