#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.DatabaseExplorer;
using dnp.DomainModeler;
using dnp.DomainModeler.Configuration;
using dnp.Framework.Reflection;
using MbUnit.Framework;
using Relationship=dnp.DomainModeler.Relationship;

namespace dnp.DomainModeler.Test
{
    [TestFixture]
    public class RelationshipTest
    {
        private Domain _domain = null;

        [TestFixtureSetUp]
        public void Setup()
        {
            DomainModelDefinition def =
                Serializer.DeserializeFromResource<DomainModelDefinition>("ReferenceSchema-DomainDefinition.xml");
            _domain = new Domain(def);
        }

        [Test]
        public void Name()
        {
            Entity entity = _domain.Entities["Car"];
            Assert.AreEqual("Owner", entity.Relationships()[0].Name);
            Assert.AreEqual("PreviousOwner", entity.Relationships()[1].Name);
            Assert.AreEqual("DealerName", entity.Relationships()[2].Name);
        }

        [Test]
        public void Source()
        {
            Entity entity = _domain.Entities["Car"];
            Assert.AreEqual("FK_Car_Person", entity.Relationships()[0].Source.Name);
            Assert.AreEqual("FK_Car_Person1", entity.Relationships()[1].Source.Name);
            Assert.AreEqual("FK_Car_Dealer", entity.Relationships()[2].Source.Name);
        }

        [Test]
        public void Domain()
        {
            Relationship relationship = _domain.Entities["Car"].Relationships()[0];
            Assert.IsNotNull(relationship.Domain);
        }

        [Test]
        public void Visibility()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual(true, entity.Relationships()[0].Visible);
            Assert.AreEqual(false, entity.Relationships()[1].Visible);
        }

        [Test]
        public void Parent()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual(entity.Name, entity.Relationships()[0].Parent.Name);

            entity = _domain.Entities["Student"];
            Assert.AreEqual(entity.Name, entity.Relationships()[0].Parent.Name);
            Assert.AreEqual(entity.Name, entity.Relationships()[1].Parent.Name);
        }

        [Test]
        public void Properties()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual(2, entity.Relationships().Count);
            Assert.AreEqual("UniversityID", entity.Relationships()[0].Properties[0].Source.Name);
            Assert.AreEqual(1, entity.Relationships()[0].Properties.Count);
            Assert.AreEqual("CourseID", entity.Relationships()[1].Properties[0].Source.Name);
            Assert.AreEqual(1, entity.Relationships()[1].Properties.Count);

            entity = _domain.Entities["Car"];
            Assert.AreEqual(3, entity.Relationships().Count);
            Assert.AreEqual(1, entity.Relationships()[0].Properties.Count);
            Assert.AreEqual("OwnerID", entity.Relationships()[0].Properties[0].Source.Name);

            Assert.AreEqual(2, entity.Relationships()[2].Properties.Count);
            Assert.AreEqual("DealerName", entity.Relationships()[2].Properties[0].Source.Name);
            Assert.AreEqual("DealerStateID", entity.Relationships()[2].Properties[1].Source.Name);
        }

        [Test]
        public void RelatedEntity()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual("University", entity.Relationships()[0].RelatedEntity.Name);

            entity = _domain.Entities["Student"];
            Assert.AreEqual("Person", entity.Relationships()[0].RelatedEntity.Name);
            Assert.AreEqual("Course", entity.Relationships()[1].RelatedEntity.Name);
            Assert.AreEqual("University", entity.Relationships()[2].RelatedEntity.Name);
        }

        [Test]
        public void IntermediateRelationship()
        {
            // Course
            Entity entity = _domain.Entities["Course"];
            dnp.DomainModeler.RelationshipCollection manyToManyRelationships = entity.Relationships(RelationshipTypeEnum.ManyToMany);
            Assert.AreEqual("Student", manyToManyRelationships[0].RelatedEntity.Name);
            Assert.IsNotNull(manyToManyRelationships[0].IntermediateRelationship);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.Source.FullName);
            Assert.AreEqual("dbo.Course", manyToManyRelationships[0].IntermediateRelationship.LeftRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.LeftRelationship.ForeignKey.TableFullName);
            Assert.AreEqual("dbo.Student", manyToManyRelationships[0].IntermediateRelationship.RightRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.RightRelationship.ForeignKey.TableFullName);

            Assert.AreEqual(1, manyToManyRelationships[0].Parent.BusinessKey.Count);
            Assert.AreEqual(2, manyToManyRelationships[0].RelatedEntity.BusinessKey.Count);


            // University (other side of above)
            entity = _domain.Entities["Student"];
            manyToManyRelationships = entity.Relationships(RelationshipTypeEnum.ManyToMany);
            // Course
            Assert.AreEqual("Course", manyToManyRelationships[0].RelatedEntity.Name);
            Assert.IsNotNull(manyToManyRelationships[0].IntermediateRelationship);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.Source.FullName);
            Assert.AreEqual("dbo.Student", manyToManyRelationships[0].IntermediateRelationship.LeftRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.LeftRelationship.ForeignKey.TableFullName);
            Assert.AreEqual("dbo.Course", manyToManyRelationships[0].IntermediateRelationship.RightRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.StudentCourseLink", manyToManyRelationships[0].IntermediateRelationship.RightRelationship.ForeignKey.TableFullName);

            // University
            Assert.AreEqual("University", manyToManyRelationships[01].RelatedEntity.Name);
            Assert.IsNotNull(manyToManyRelationships[1].IntermediateRelationship);
            Assert.AreEqual("dbo.University_Students", manyToManyRelationships[1].IntermediateRelationship.Source.FullName);
            Assert.AreEqual("dbo.Student", manyToManyRelationships[1].IntermediateRelationship.LeftRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.University_Students", manyToManyRelationships[1].IntermediateRelationship.LeftRelationship.ForeignKey.TableFullName);
            Assert.AreEqual("dbo.University", manyToManyRelationships[1].IntermediateRelationship.RightRelationship.PrimaryKey.TableFullName);
            Assert.AreEqual("dbo.University_Students", manyToManyRelationships[1].IntermediateRelationship.RightRelationship.ForeignKey.TableFullName);
        }

        [Test]
        public void RelatedEntityProperties()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual(1, entity.Relationships()[0].RelatedEntityProperties.Count);
            Assert.AreEqual("UniversityID", entity.Relationships()[0].RelatedEntityProperties[0].Source.Name);
            Assert.AreEqual(1, entity.Relationships()[1].RelatedEntityProperties.Count);
            Assert.AreEqual("StudentID", entity.Relationships()[1].RelatedEntityProperties[0].Source.Name);

            entity = _domain.Entities["Car"];
            Assert.AreEqual(1, entity.Relationships()[0].RelatedEntityProperties.Count);
            Assert.AreEqual("PersonID", entity.Relationships()[0].RelatedEntityProperties[0].Source.Name);
            Assert.AreEqual(2, entity.Relationships()[2].RelatedEntityProperties.Count);
            Assert.AreEqual("DealerName", entity.Relationships()[2].RelatedEntityProperties[0].Source.Name);
            Assert.AreEqual("StateID", entity.Relationships()[2].RelatedEntityProperties[1].Source.Name);
        }

        [Test]
        public void RelationshipType()
        {
            Entity entity = _domain.Entities["Course"];
            Assert.AreEqual(2, entity.Relationships().Count);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, entity.Relationships()[0].RelationshipType);
            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, entity.Relationships()[1].RelationshipType);


            entity = _domain.Entities["Car"];
            Assert.AreEqual(3, entity.Relationships().Count);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, entity.Relationships()[0].RelationshipType);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, entity.Relationships()[1].RelationshipType);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, entity.Relationships()[2].RelationshipType);
        }
    }
}