#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;

namespace dnp.DomainModeler
{
    [Serializable]
    public class BaseEntityClass
    {
        private string _namespace = "";
        private string _className = "";
        private BusinessEntityAttributeCollection _attributes = new BusinessEntityAttributeCollection();
        private BusinessEntityAttributeCollection _properties;
        private BusinessEntityAttributeCollection _methods;

        public string ClassName
        {
            get { return _className; }
            set { _className = value; }
        }

        public string Namespace
        {
            get { return _namespace; }
            set { _namespace = value; }
        }

        public string FullClassName
        {
            get { return Model.FixNamespace(this.Namespace + "." + this.ClassName); }
        }
        public BusinessEntityAttributeCollection Attributes
        {
            get { return _attributes; }
            set { _attributes = value; }
        }

        /// <summary>
        /// This method determines if the base entity supports a predefined property role
        /// </summary>
        /// <param name="role">The property role to be tested</param>
        /// <returns>true if entity supports role otherwise false</returns>
        public bool SupportsRole(AttributeRoleEnum role)
        {
            foreach(BusinessEntityAttribute p in this.Attributes)
                if(p.AttributeRole == role)
                    return true;

            return false;
        }

        /// <summary>
        /// Returns all defined attributes that are methods
        /// </summary>
        /// <returns></returns>
        public BusinessEntityAttributeCollection Methods
        {
            get
            {
                if (_methods == null)
                {
                    _methods = new BusinessEntityAttributeCollection();
                    foreach (BusinessEntityAttribute a in this.Attributes)
                        switch (a.AttributeRole)
                        {
                            case AttributeRoleEnum.Field:
                            case AttributeRoleEnum.PrimaryKey:
                            case AttributeRoleEnum.ForeignKey:
                            case AttributeRoleEnum.ConcurrencyKey:
                            case AttributeRoleEnum.CreatedBy:
                            case AttributeRoleEnum.ModifiedBy:
                            case AttributeRoleEnum.CreatedDate:
                            case AttributeRoleEnum.ModifiedDate:
                                break;
                            case AttributeRoleEnum.IsValid:
                                _methods.Add(a);
                                break;
                            default:
                                break;
                        }
                }
                return _methods;
            }
        }

        /// <summary>
        /// Returns all defined attributes that are properties
        /// </summary>
        /// <returns></returns>
        public BusinessEntityAttributeCollection Properties
        {
            get
            {
                if (_properties == null)
                {
                    _properties = new BusinessEntityAttributeCollection();
                    foreach (BusinessEntityAttribute a in this.Attributes)
                        switch (a.AttributeRole)
                        {
                            case AttributeRoleEnum.Field:
                            case AttributeRoleEnum.PrimaryKey:
                            case AttributeRoleEnum.ForeignKey:
                            case AttributeRoleEnum.ConcurrencyKey:
                            case AttributeRoleEnum.CreatedBy:
                            case AttributeRoleEnum.ModifiedBy:
                            case AttributeRoleEnum.CreatedDate:
                            case AttributeRoleEnum.ModifiedDate:
                                _properties.Add(a);
                                break;
                            case AttributeRoleEnum.IsValid:
                                break;
                            default:
                                break;
                        }
                }
                return _properties;
            }
        }

        /// <summary>
        /// Retrieves the business entity property for the supplied role.
        /// </summary>
        /// <param name="role">The property role to be tested</param>
        /// <returns>If role is supported returns an instance of BusinessEntityAttribute otherwise returns null</returns>
        public BusinessEntityAttribute PropertyRole(AttributeRoleEnum role)
        {
            foreach (BusinessEntityAttribute p in this.Attributes)
                if (p.AttributeRole == role)
                    return p;

            return null;
        }

        public bool HasRole(AttributeRoleEnum role, Property property)
        {
            return this.SupportsRole(role) && this.PropertyRole(role).Name == property.Name;
        }
    }
}