#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Xml.Serialization;

namespace dnp.DomainModeler.Configuration
{
    [Serializable]
    [XmlRoot("DomainDefinition")]
    public class DomainModelDefinition
    {
        private string _modelName = "";
        private string _clientName = "";
        private string _modelConnectionString = "";
        private string _ignoredTablesForManyToManyCalcs = "";

        private bool _resolveLooseRelationships;
        private BaseEntityClass _baseEntityClass = new BaseEntityClass();
        private EntityDefinitionCollection _Entities = new EntityDefinitionCollection();

        private ManyToManyRuleDefinitionCollection _manyToManyRuleDefinition =
            new ManyToManyRuleDefinitionCollection();

        public string IgnoredTablesForManyToManyCalcs
        {
            get { return _ignoredTablesForManyToManyCalcs; }
            set { _ignoredTablesForManyToManyCalcs = value; }
        }

        /// <summary>
        /// The name of the client who's domain is being modeled. This information can then be used in copywright notices.
        /// </summary>
        public string ClientName
        {
            get { return _clientName; }
            set { _clientName = value; }
        }

        /// <summary>
        /// This is the name of the model that is being modeled. This can be used for documentation purposes.
        /// </summary>
        public string ModelName
        {
            get { return _modelName; }
            set { _modelName = value; }
        }

        /// <summary>
        /// This is the connection string that is used to connect to the database to retrieve the model schema.
        /// </summary>
        public string ModelConnectionString
        {
            get { return _modelConnectionString; }
            set { _modelConnectionString = value; }
        }

        /// <summary>
        /// Some enities have relationships which can be inferred by intermidiary relationships. When selected the system will attempt to resolve these 
        /// 'extra' relationships otherwise if false no attempt will be made to resolve these relationships.
        /// </summary>
        public bool ResolveLooseRelationships
        {
            get { return _resolveLooseRelationships; }
            set { _resolveLooseRelationships = value; }
        }


        /// <summary>
        /// This property defines a collection of fields that are to be excluded when calcualting Many to Many relationships.
        /// </summary>
        public ManyToManyRuleDefinitionCollection ManyToManyRuleDefinition
        {
            get { return _manyToManyRuleDefinition; }
            set { _manyToManyRuleDefinition = value; }
        }

        /// <summary>
        /// Defines the class to be used as a base (super) class for all entities. This definition is used by
        /// scripts to determine how to create entities and other related information.
        /// </summary>
        public BaseEntityClass BaseEntityClass
        {
            get { return _baseEntityClass; }
            set { _baseEntityClass = value; }
        }

        public EntityDefinitionCollection Entities
        {
            get { return _Entities; }
            set { _Entities = value; }
        }
    }
}