#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Scripting;
using dnp.ServiceModel;

namespace dnp.DomainModeler
{
    public abstract class Model
    {
        private Domain _domain;
        private string _version = "1.0.0";

        protected Model()
        {
        }

        protected Model(ModelDefinition definition)
        {
            Initialize(definition);
        }

        protected virtual void Initialize(ModelDefinition definition)
        {
            _domain = new Domain(definition.DomainDefinition);
            _definition = definition;
            // Associate the domain with this model
            _domain.Model = this;
        }

        protected ModelDefinition _definition = new ModelDefinition();

        /// <summary>
        /// The root path for clases to be created in
        /// </summary>
        public string RootPath
        {
            get { return Naming.EvaluateNamingExpression(_definition.RootPath, this); }
        }

        /// <summary>
        /// The name of the client who's domain is being modeled. This information can then be used in copywright notices.
        /// </summary>
        public string ClientName
        {
            get { return Naming.EvaluateNamingExpression(_definition.ClientName, this); }
        }

        /// <summary>
        /// This is the name of the model that is being modeled. This can be used for documentation purposes.
        /// </summary>
        public string ModelName
        {
            get { return Naming.EvaluateNamingExpression(_definition.ModelName, this); }
        }

        /// <summary>
        /// This is the root namespace of the model. This namespace is used as the prefix for all other namespaces.
        /// </summary>
        public string RootNamespace
        {
            get { return Naming.EvaluateNamingExpression(_definition.RootNamespace, this); }
        }

        /// <summary>
        /// The standard copyright notice for the model.
        /// </summary>
        public string CopyrightNotice
        {
            get { return Naming.EvaluateNamingExpression(_definition.CopyrightNotice, this); }
        }

        /// <summary>
        /// The version of the model, usually used to version model output.
        /// </summary>
        public string Version
        {
            get { return _version; }
            set { _version = value; }
        }

        public Domain Domain
        {
            get { return _domain; }
        }

        /// <summary>
        /// Given a namespace will remove any double periods, trailing and leading periods to ensure a safe namespace.
        /// </summary>
        /// <param name="namespaceDefinition">namespace to fix</param>
        /// <returns></returns>
        public static string FixNamespace(string namespaceDefinition)
        {
            string newNamespace = namespaceDefinition.Trim('.'); // Remove any trailing and leading periods
            newNamespace = newNamespace.Replace("..", "."); // remove any double periods.

            return newNamespace;
        }
    }
}