#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using dnp.DomainModeler.Configuration;
using dnp.Framework.Database.DatabaseExplorer;

namespace dnp.DomainModeler
{
    public class PrimaryKey 
    {
        private DomainModelDefinition _definition;
        private Constraint _source;
        private PropertyCollection _properties = new PropertyCollection();
        private Entity _parent;

        public PrimaryKey(Constraint source, Entity parent, DomainModelDefinition definition)
        {
            _definition = definition;
            _source = source;
            _parent = parent;

            // Some tables don't have primary keys
            if (source != null)
            {
                // Check that the EntityBaseClass has only 1  or 0 primary keys defined.
                int primaryKeyColumns = 0;
                foreach (BusinessEntityAttribute p in _definition.BaseEntityClass.Attributes)
                    if (p.AttributeRole == AttributeRoleEnum.PrimaryKey)
                        primaryKeyColumns += 1;

                if (primaryKeyColumns > 1)
                    throw new ArgumentException("Base Entity Class can only have 1 property defined as a primary key.");

                // Now define the properties that make up the primary key.
                foreach (TableColumn c in this.Source.Columns)
                    _properties.Add(this.GetProperty(c.Name));
            }
        }

        /// <summary>
        /// As the Properties collection is keyed by the name supplied as part of the configuration file, the database name/alias may  not match the defined one
        /// in the config. As a result we need to search for the property the hard way!
        /// </summary>
        private Property GetProperty(string sourceName)
        {
            foreach (Property p in this.Parent.Properties(PropertyFilterEnum.FieldPrimaryKey))
                if (p.Source.Name == sourceName)
                    return p;

            throw new ArgumentException("Unable to find property with a souce name of " + sourceName);
        }
        public Constraint Source
        {
            get { return _source; }
        }


        public PropertyCollection Properties
        {
            get { return _properties; }
        }

        public Entity Parent
        {
            get { return _parent; }
        }
    }
}
