#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using dnp.DomainModeler.Configuration;
using dnp.Framework.Database.DatabaseExplorer;

namespace dnp.DomainModeler
{
    public class Relationship : DomainIdentifierBase<Entity>
    {
        private RelationshipDefinition _definition;
        private Entity _relatedEntity;
        private PropertyCollection _properties;
        private PropertyCollection _relatedEntityProperties;
        private readonly IntermediateRelationship _intermediateRelationship;

        public Relationship(Framework.Database.DatabaseExplorer.Relationship source, Entity parent, RelationshipDefinition definition)
            : base(source, definition.Name, definition.Visible, parent)
        {
            _definition = definition;
            if (source.IntermediateTable != null)
                _intermediateRelationship = new IntermediateRelationship(source.IntermediateTable, source.LeftConstraint.Name, true, this);
            else
                _intermediateRelationship = null;
        }

        public new Framework.Database.DatabaseExplorer.Relationship Source
        {
            get { return base.Source as Framework.Database.DatabaseExplorer.Relationship; }
        }
        public RelationshipTypeEnum RelationshipType
        {
            get { return this.Source.RelationshipType; }
        }

        public PropertyCollection Properties
        {
            get
            {
                if (_properties == null)
                {
                    _properties = new PropertyCollection();

                    foreach (Property p in this.Parent.Properties(PropertyFilterEnum.All, VisibilityEnum.All))
                        if (this.Source.LeftConstraint.Columns.Contains(p.Source.Name))
                            _properties.Add(p);
                }
                return _properties;
            }
        }

        public PropertyCollection RelatedEntityProperties
        {
            get
            {
                if (_relatedEntityProperties == null)
                {
                    _relatedEntityProperties = new PropertyCollection();
                    foreach (Property p in this.RelatedEntity.Properties(PropertyFilterEnum.All, VisibilityEnum.All))
                        if (this.Source.RightConstraint.Columns.Contains(p.Source.Name))
                            _relatedEntityProperties.Add(p);
                }
                if (_relatedEntityProperties.Count == 0)
                    throw new ArgumentException("No related properties found for relationship" + this.Name);
                return _relatedEntityProperties;
            }
        }

        public Entity RelatedEntity
        {
            get
            {
                if (_relatedEntity == null)
                {
                    foreach (Entity e in this.Domain.Entities)
                        if (e.Source.Name == this.Source.RightConstraint.TableName)
                        {
                            _relatedEntity = e;
                            break;
                        }
                }
                return _relatedEntity;
            }
        }
        public override Domain Domain
        {
            get { return this.Parent.Parent; }
        }

        public IntermediateRelationship IntermediateRelationship
        {
            get { return _intermediateRelationship; }
        }
    }
}