#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
#region Using Statements

using System.Text.RegularExpressions;
using dnp.Framework.Database.DataProvider;
using dnp.Framework.IO;

#endregion

namespace dnp.Framework.Database.InformationSchema
{
    public class SQLServerProvider : InformationSchemaProviderBase
    {
        private string _databaseName = "";
        private IDataProvider _provider;

        #region Constructors

        public SQLServerProvider(string connectionString)
        {
            this.ConnectionString = connectionString;
        }

        public SQLServerProvider()
        {
        }

        #endregion

        protected override IDataProvider DataProvider()
        {
            if (_provider == null)
            {
                _provider = new DataProvider.SQLServerProvider();
                _provider.ConnectionString = this.ConnectionString;
                _databaseName = this.DetermineDatabaseName();
            }
            return _provider;
        }

        private string DetermineDatabaseName()
        {
            Regex r = new Regex(@"Catalog\s*=\s*(\w+);?");
            Match m = r.Match(this.ConnectionString);
            if (m.Success)
                return m.Groups[1].ToString();
            else
                return "";
        }

        #region Information Schema SQL Definitions

        public override string DatabaseName
        {
            get { return _databaseName; }
        }

        protected override string ColumnSchemaSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.Columns.sql");
        }

        protected override string ConstraintSchemaSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.Constraints.sql");
        }

        protected override string ForeignConstraintSchemaSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.ForeignConstraints.sql");
        }

        protected override string IndexSchemaSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.Indexes.sql");
        }

        protected override string ViewColumnSchemaSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.ViewColumns.sql");
        }

        protected override string ExtendedProperties()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.ExtendedProperties.sql");
        }

        protected override string ViewTextSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.View.Text.sql");
        }


        protected override string RoutineColumnsSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.RoutineColumns.sql");
        }

        protected override string RoutineParametersSQL()
        {
            return EmbeddedResources.GetResourceString("InformationSchema.RoutineParameters.sql");
        }      
        
        #endregion

    }
}