#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections;
using CodeSmith.Engine;

namespace dnp.Framework.Scripting
{
    /// <summary>
    /// This class provides operations specific to code generation templates. The methods
    /// are abstracted to support multi-vendor code-generation.
    /// </summary>
    public class Template
    {
        private static Hashtable s_cachedTemplates = new Hashtable();

        public static CodeTemplate GetTemplate(string relativePath, string fileName, CodeTemplate instance)
        {
            return GetTemplate(relativePath, fileName, instance, null);
        }

        public static CodeTemplate GetTemplate(string relativePath, string fileName, CodeTemplate instance, string version)
        {
            string templateFileName = "";

            if (!string.IsNullOrEmpty(version))
            {
                templateFileName = string.Format(@"{0}\{1}\{2}\{3}", instance.CodeTemplateInfo.DirectoryName, version, relativePath, fileName);
                if (!System.IO.File.Exists(templateFileName))
                    templateFileName = string.Format(@"{0}\{1}\{2}", instance.CodeTemplateInfo.DirectoryName, relativePath, fileName);
            }
            else
                templateFileName = string.Format(@"{0}\{1}\{2}", instance.CodeTemplateInfo.DirectoryName, relativePath, fileName);

            CodeTemplate objTemplate;
            objTemplate = s_cachedTemplates[templateFileName] as CodeTemplate;
            if (objTemplate == null)
            {
                if (!System.IO.File.Exists(templateFileName))
                    throw new ArgumentException("The template with this filename could not be found: " + templateFileName);

                CodeTemplateCompiler objCompiler = new CodeTemplateCompiler(templateFileName);
                objCompiler.Compile();
                if (objCompiler.Errors.Count == 0)
                {
                    objTemplate = objCompiler.CreateInstance();
                    s_cachedTemplates.Add(templateFileName, objTemplate);
                }
            }
            instance.CopyPropertiesTo(objTemplate);
            return objTemplate;
        }

        /// <summary>
        /// Render the contents of the template to the specified file. If the file exists already, the file won't be written.
        /// </summary>
        /// <param name="fileName">Filename to render the template to</param>
        /// <param name="template">The template instance to render</param>
        public static void RenderToFile(string fileName, CodeTemplate template)
        {
            RenderToFile(fileName, template, false);
        }

        /// <summary>
        /// Render the contents of the template to the specified file.
        /// </summary>
        /// <param name="fileName">Filename to render the template to</param>
        /// <param name="template">The template instance to render</param>
        /// <param name="overwriteFile">Determines if the file should be overritten if present</param>
        public static void RenderToFile(string fileName, CodeTemplate template, bool overwriteFile)
        {
            if (!overwriteFile && System.IO.File.Exists(fileName))
            {
                // If the file already exists don't overwrite it!
                return;
            }

            // Check that the path doesn't already exist
            string filePath = System.IO.Path.GetDirectoryName(fileName);
            if (!System.IO.Directory.Exists(filePath))
                System.IO.Directory.CreateDirectory(filePath);

            template.RenderToFile(fileName, true);
        }
        public static void ResetCache()
        {
            s_cachedTemplates = new Hashtable();
        }
    }
}