#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections.Generic;
using dnp.Framework.Reflection;
using dnp.Framework.UnitTest.Collections.Shams;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Collections
{
    [TestFixture]
    public class MultiKeyedCollectionSingleKeyTest
    {
        [Test]
        public void AddItem()
        {
            TableCollection tables = new TableCollection();

            Assert.AreEqual(0, tables.Count);

            tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            Assert.AreEqual("Table1", tables[0].Name);
        }

        [Test]
        public void ItemKeyFound()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual("Table2", tables["Table2"].Name);
            Assert.AreEqual("Table1", tables["Table1"].Name);
            Assert.AreEqual("Table4", tables["Table4"].Name);
        }

        [Test]
        [ExpectedException(typeof(KeyNotFoundException))]
        public void ItemKeyNotFound()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(null, tables["Table99"].Name);
        }

        [Test]
        public void InsertItemAt()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Insert(3, new Table("UnitTest1", "Table5"));
            Assert.AreEqual(5, tables.Count);
            Assert.AreEqual("Table5", tables[3].Name);
            Assert.AreEqual("Table5", tables["Table5"].Name);
        }

        [Test]
        public void RemoveItemAt()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.RemoveAt(2);
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table2", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table2", tables["Table2"].Name);
            Assert.AreEqual("Table4", tables["Table4"].Name);
        }

        [Test]
        public void Remove()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Remove("Table3");
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table2", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table2", tables["Table2"].Name);
            Assert.AreEqual("Table4", tables["Table4"].Name);
        }

        [Test]
        public void RemoveII()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Remove("Table2");
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table3", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table3", tables["Table3"].Name);
            Assert.AreEqual("Table4", tables["Table4"].Name);
        }

        [Test]
        public void ContainsKey()
        {
            TableCollection tables = this.PopulateTables();

            Assert.IsTrue(tables.Contains("Table4"));
            Assert.IsFalse(tables.Contains("Table99"));
        }

        [Test]
        public void Key()
        {
            TableCollection tables = this.PopulateTables();

            Assert.AreEqual(1, tables.Key.Count);
            Assert.AreEqual("Name", tables.Key[0]);
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DuplicateKey()
        {
            TableCollection tables = this.PopulateTables();
            // This object already exists in the collection so we should get an exception!
            tables.Add(new Table("UnitTest1", "Table1"));
        }

        [Test]
        public void Serialization()
        {

            TableCollection tables = new TableCollection();
            tables.Add(new Table("UnitTest1", "Table1"));
            tables.Add(new Table("UnitTest1", "Table2"));
            tables.Add(new Table("UnitTest1", "Table3"));
            tables.Add(new Table("UnitTest2", "Table4"));

            string xml = Serializer.Serialize(tables);

            Assert.IsNotNull(xml);
        }

        private TableCollection PopulateTables()
        {
            TableCollection tables = new TableCollection();
            tables.Add(new Table("UnitTest1", "Table1"));
            tables.Add(new Table("UnitTest1", "Table2"));
            tables.Add(new Table("UnitTest1", "Table3"));
            tables.Add(new Table("UnitTest2", "Table4"));
            return tables;
        }
    }

}