#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections.Generic;
using dnp.Framework.UnitTest.Collections.Shams;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Collections
{
    [TestFixture]
    public class MultiKeyedCollectionTest
    {
        [Test]
        [ExpectedException(typeof(KeyNotFoundException))]
        public void InvalidKey()
        {
            Shams.InvalidCollection c = new InvalidCollection();

            // An exception should have been thrown...
        }
        [Test]
        public void AddItem()
        {
            DatabaseTableCollection tables = new DatabaseTableCollection();

            Assert.AreEqual(0, tables.Count);

            tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            Assert.AreEqual("Table1", tables[0].Name);
        }

        [Test]
        public void ItemKeyFound()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual("Table2", tables["UnitTest1", "Table2"].Name);
            Assert.AreEqual("Table1", tables["UnitTest1", "Table1"].Name);
            Assert.AreEqual("Table4", tables["UnitTest2", "Table4"].Name);
        }

        [Test]
        [ExpectedException(typeof(KeyNotFoundException))]
        public void ItemKeyNotFound()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(null, tables["UnitTest1", "Table99"].Name);
        }

        [Test]
        public void InsertItemAt()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Insert(3, new Table("UnitTest1", "Table5"));
            Assert.AreEqual(5, tables.Count);
            Assert.AreEqual("Table5", tables[3].Name);
            Assert.AreEqual("Table5", tables["UnitTest1", "Table5"].Name);
        }

        [Test]
        public void RemoveItemAt()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.RemoveAt(2);
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table2", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table2", tables["UnitTest1", "Table2"].Name);
            Assert.AreEqual("Table4", tables["UnitTest2", "Table4"].Name);
        }

        [Test]
        public void Remove()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Remove("UnitTest1", "Table3");
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table2", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table2", tables["UnitTest1", "Table2"].Name);
            Assert.AreEqual("Table4", tables["UnitTest2", "Table4"].Name);
        }

        [Test]
        public void RemoveII()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(4, tables.Count);
            tables.Remove("UnitTest1", "Table2");
            Assert.AreEqual(3, tables.Count);
            Assert.AreEqual("Table3", tables[1].Name);
            Assert.AreEqual("Table4", tables[2].Name);
            Assert.AreEqual("Table3", tables["UnitTest1", "Table3"].Name);
            Assert.AreEqual("Table4", tables["UnitTest2", "Table4"].Name);
        }

        [Test]
        public void ContainsKey()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.IsTrue(tables.Contains("UnitTest2", "Table4"));
            Assert.IsFalse(tables.Contains("UnitTest1", "Table99"));
        }

        [Test]
        public void Key()
        {
            DatabaseTableCollection tables = this.PopulateTables();

            Assert.AreEqual(2, tables.Key.Count);
            Assert.AreEqual("Schema", tables.Key[0]);
            Assert.AreEqual("Name", tables.Key[1]);
        }

        [Test]
        [ExpectedException(typeof (ArgumentException))]
        public void DuplicateKey()
        {
            DatabaseTableCollection tables = this.PopulateTables();
            // This object already exists in the collection so we should get an exception!
            tables.Add(new Table("UnitTest1", "Table1"));
        }

        private DatabaseTableCollection PopulateTables()
        {
            DatabaseTableCollection tables = new DatabaseTableCollection();
            tables.Add(new Table("UnitTest1", "Table1"));
            tables.Add(new Table("UnitTest1", "Table2"));
            tables.Add(new Table("UnitTest1", "Table3"));
            tables.Add(new Table("UnitTest2", "Table4"));
            return tables;
        }
    }
}