#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System.Data;
using System.Transactions;
using dnp.Framework.Database.DataProvider;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DataProvider
{
    [TestFixture]
    public class DataProviderFactoryTest
    {
        /// <summary>
        /// Tests that the configuration file works and that an instance of a data provider can be created dynamically.
        /// </summary>
        [Test]
        public void GetDataProvider()
        {
            IDataProvider dataProvider;

            dataProvider = DataProviderFactory.GetDataProvider("ReferenceSchema");

            Assert.AreEqual("dnp.Framework.Database.DataProvider.SQLServerProvider", dataProvider.ToString());
        }

        [Test]
        public void ExecuteCommand()
        {
            IDataProvider dataProvider;
            dataProvider = DataProviderFactory.GetDataProvider("ReferenceSchema");

            // Now define a command to execute against the database
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = "Insert into Home VALUES('My Address')";
            cmd.CommandType = CommandType.Text;

            using (TransactionScope scope = new TransactionScope())
            {
                dataProvider.Execute(cmd);

                // Because we dont want the transaction to commit we wont complete the transaction.
                // scope.Complete();
            }

            // The assertion is that no exception was raised.
        }
    }
}