#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.DatabaseExplorer;
using dnp.Framework.Database.InformationSchema;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DatabaseExplorer
{
    [TestFixture]
    public class DatabaseTest
    {
        const string _connectionString = @"Data Source=(local)\SQLExpress;Initial Catalog=ReferenceSchema;Integrated Security=true;Connection Timeout=1;";
        [Test]
        public void Constructor_DatabaseName()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            Assert.AreEqual("dnp.Framework.Database.InformationSchema.SQLServerProvider", database.Provider.ToString());
            this.CommonDatabaseTests(database);
        }

        [Test]
        public void Constructor_SQLProviderInstance()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database =
                new Framework.Database.DatabaseExplorer.Database(new SQLServerProvider(),
                                                       _connectionString,
                                                       null);

            Assert.AreEqual("dnp.Framework.Database.InformationSchema.SQLServerProvider", database.Provider.ToString());
            this.CommonDatabaseTests(database);
        }

        [Test]
        public void Constructor_SQLProviderName()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database =
                new Framework.Database.DatabaseExplorer.Database("SQLServer",
                                                       _connectionString,
                                                       null);

            Assert.AreEqual("dnp.Framework.Database.InformationSchema.SQLServerProvider", database.Provider.ToString());
            this.CommonDatabaseTests(database);
        }

        [Test]
        public void Schemas()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            Assert.AreEqual(2, database.Schemas.Count, "Database.Schemas.Count");
            Assert.AreEqual("dbo", database.Schemas[0].Name, "Database.Schemas[0].Name");
            Assert.AreEqual("test", database.Schemas[1].Name, "Database.Schemas[1].Name");

            Assert.AreEqual(17, database.Schemas[0].Tables.Count, "Database.Schema.Tables.Count");
            Assert.AreEqual(4, database.Schemas[0].Views.Count, "Database.Schema.Views.Count");

            Assert.AreEqual(1, database.Schemas[1].Tables.Count, "Database.Schema.Tables.Count");
            Assert.AreEqual(0, database.Schemas[1].Views.Count, "Database.Schema.Views.Count");
        }

        [Test]
        public void Indexes_Car()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            Table t = database.Tables["dbo.Car"];

            Assert.AreEqual(4, t.Indexes.Count, "Table.Indexes.Count");
            Index i = t.Indexes["IXU_MakeModel"];
            Assert.AreEqual("IXU_MakeModel", i.Name);
            Assert.AreEqual("Make", i.Columns[0].Name);
            Assert.AreEqual("Model", i.Columns[1].Name);
            Assert.IsFalse(i.IsClustered);
            Assert.IsTrue(i.IsUnique);
            Assert.IsFalse(i.IsPrimaryKey);

            i = t.Indexes["PK_Car"];
            Assert.AreEqual("PK_Car", i.Name);
            Assert.AreEqual("CarID", i.Columns[0].Name);
            Assert.IsFalse(i.IsClustered);
            Assert.IsTrue(i.IsUnique);
            Assert.IsTrue(i.IsPrimaryKey);

            i = t.Indexes["IX_DealerStateID"];
            Assert.AreEqual("IX_DealerStateID", i.Name);
            Assert.AreEqual("DealerStateID", i.Columns[0].Name);
            Assert.IsFalse(i.IsClustered);
            Assert.IsFalse(i.IsUnique);
            Assert.IsFalse(i.IsPrimaryKey);

            i = t.Indexes["IX_Car_OwnerID"];
            Assert.AreEqual("IX_Car_OwnerID", i.Name);
            Assert.AreEqual("OwnerID", i.Columns[0].Name);
            Assert.IsFalse(i.IsClustered);
            Assert.IsFalse(i.IsUnique);
            Assert.IsFalse(i.IsPrimaryKey);
        }

        [Test]
        public void Views()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            Assert.AreEqual(4, database.Views.Count, "Database.Views.Count");

            Assert.AreEqual("v_AllDataTypes", database.Views[0].Name, "Database.Views[0].Name");
            Assert.AreEqual("v_Course", database.Views[1].Name, "Database.Views[0].Name");
            Assert.AreEqual("v_CourseStudentCount", database.Views[2].Name, "Database.Views[2].Name");
            Assert.AreEqual("v_CourseStudents", database.Views[3].Name, "Database.Views[1].Name");
        }

        [Test]
        public void OrderedTables_InsertOrder()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            TableCollection orderedList = database.OrderedTables(TableOrderEnum.InsertOrder);

            string[] expected = {
                                    "AllDataTypes", "Country", "Descriptions", "Home", "MultipleTableNameRelationshipTest", "SingleTable", "State", "University", "Course", "Person",
                                    "Region", "This_That_TheOther_EntityTable", "Dealer", "Car", "Student", "PreviousStudent", "StudentCourseLink", "University_Students"
                                };

            for (int i = 0; i < orderedList.Count; i++)
            {
                Assert.AreEqual(expected[i], orderedList[i].Name);
            }

            Assert.AreEqual(database.Tables.Count, orderedList.Count);

        }

        [Test]
        public void OrderedTables_DeleteOrder_Count()
        {
            Framework.Database.DatabaseExplorer.Database database;
            database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);

            TableCollection orderedList = database.OrderedTables(TableOrderEnum.DeleteOrder);

            string[] expected = {
                                    "University_Students", "StudentCourseLink", "PreviousStudent", "Student", "Car", "Dealer", "This_That_TheOther_EntityTable", "Region", "Person",
                                    "Course", "University", "State", "SingleTable", "MultipleTableNameRelationshipTest", "Home", "Descriptions", "Country", "AllDataTypes"

                                };

            for (int i = 0; i < orderedList.Count; i++)
            {
                Assert.AreEqual(expected[i], orderedList[i].Name);
            }

            Assert.AreEqual(database.Tables.Count, orderedList.Count);
        }

        private void CommonDatabaseTests(Framework.Database.DatabaseExplorer.Database db)
        {
            Assert.AreEqual(18, db.Tables.Count);
        }
    }
}