#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion

using dnp.Framework.Database.DatabaseExplorer;
using dnp.Framework.Scripting;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DatabaseExplorer
{
    [TestFixture]
    public class RelationshipTest
    {
        private Framework.Database.DatabaseExplorer.Database _database;

        [TestFixtureSetUp]
        public void Setup()
        {
            SystemFieldCollection systemFields = new SystemFieldCollection();
            // Setup the DatabaseExplorer with the system fields
            systemFields.Add(new SystemField("CreatedBy"));
            systemFields.Add(new SystemField("ModifiedBy"));
            systemFields.Add(new SystemField("CreatedDate"));
            systemFields.Add(new SystemField("ModifiedDate"));
            systemFields.Add(new SystemField("ConcurrencyID"));

            _database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", systemFields);

        }

        [Test]
        public void RelationshipType_OneToMany_Person()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Person"];
            r = t.Relationships["FK_Car_Person"];

            Assert.AreEqual(RelationshipTypeEnum.OneToMany, r.RelationshipType);
            Assert.AreEqual("Car", r.ForeignKey.TableName);
            Assert.AreEqual("Person", r.PrimaryKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_OneToMany_Country()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Country"];
            r = t.Relationships["FK_Region_Country"];

            Assert.AreEqual(RelationshipTypeEnum.OneToMany, r.RelationshipType);
            Assert.AreEqual("Region", r.ForeignKey.TableName);
            Assert.AreEqual("Country", r.PrimaryKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_ManyToMany_Student()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Student"];
            r = t.Relationships["FK_StudentCourseLink_Student"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, r.RelationshipType);
            Assert.AreEqual("Course", r.ForeignKey.TableName);
            Assert.AreEqual("Student", r.PrimaryKey.TableName);
            Assert.IsNotNull(r.IntermediateTable);
            //Assert.AreEqual("FK_StudentCourseLink_Student", r.IntermediateTable.Name);
            //Assert.AreEqual(ConstraintTypeEnum.ForeignKey, r.IntermediateTable.ConstraintType);
            Assert.AreEqual("StudentCourseLink", r.IntermediateTable.Name);
        }

        [Test]
        public void RelationshipType_ManyToMany_Course()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Course"];
            r = t.Relationships["FK_StudentCourseLink_Course"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, r.RelationshipType);
            Assert.AreEqual("Student", r.ForeignKey.TableName);
            Assert.AreEqual("Course", r.PrimaryKey.TableName);
            Assert.IsNotNull(r.IntermediateTable);
            //Assert.AreEqual("FK_StudentCourseLink_Course", r.IntermediateTable.Name);
            //Assert.AreEqual(ConstraintTypeEnum.ForeignKey, r.IntermediateTable.ConstraintType);
            Assert.AreEqual("StudentCourseLink", r.IntermediateTable.Name);
        }

        [Test]
        public void RelationshipType_ManyToMany_University()
        {

            Relationship r = null;
            Table t = _database.Tables["dbo.University"];
            r = t.Relationships["FK_University_Students_University"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, r.RelationshipType);
            Assert.AreEqual("University", r.PrimaryKey.TableName);
            Assert.AreEqual("Student", r.ForeignKey.TableName);
            Assert.IsNotNull(r.IntermediateTable);
            //Assert.AreEqual("FK_University_Students_University", r.IntermediateTable.Name);
            //Assert.AreEqual(ConstraintTypeEnum.ForeignKey, r.IntermediateTable.ConstraintType);
            Assert.AreEqual("University_Students", r.IntermediateTable.Name);
        }
        [Test]
        public void RelationshipType_ManyToOne_Car()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Car"];
            r = t.Relationships["FK_Car_Person"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, r.RelationshipType);
            Assert.AreEqual("Person", r.PrimaryKey.TableName);
            Assert.AreEqual("Car", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_ManyToOne_Dealer()
        {
            Relationship r = null;
            Table t = _database.Tables["test.Dealer"];
            r = t.Relationships["FK_Dealer_State"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, r.RelationshipType);
            Assert.AreEqual("State", r.PrimaryKey.TableName);
            Assert.AreEqual("Dealer", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_ManyToOne_RegionCountry()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Region"];
            r = t.Relationships["FK_Region_Country"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, r.RelationshipType);
            Assert.AreEqual("Country", r.PrimaryKey.TableName);
            Assert.AreEqual("Region", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_ManyToOne_RegionState()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Region"];
            r = t.Relationships["FK_Region_State"];

            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, r.RelationshipType);
            Assert.AreEqual("State", r.PrimaryKey.TableName);
            Assert.AreEqual("Region", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_OneToOne_StudentPerson()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Student"];
            r = t.Relationships["FK_Student_Person"];

            Assert.AreEqual(RelationshipTypeEnum.OneToOne, r.RelationshipType);
            Assert.AreEqual("Person", r.PrimaryKey.TableName);
            Assert.AreEqual("Student", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_OneToOne_PreviousStudentStudent()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.PreviousStudent"];
            r = t.Relationships["FK_PreviousStudent_Student"];

            Assert.AreEqual(RelationshipTypeEnum.OneToOne, r.RelationshipType);
            Assert.AreEqual("Student", r.PrimaryKey.TableName);
            Assert.AreEqual("PreviousStudent", r.ForeignKey.TableName);
            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void RelationshipType_None_AllDataTypes()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            Assert.AreEqual(0, t.Relationships.Count);
        }

        [Test]
        public void RelationshipType_Recursion_Person()
        {
            Relationship r = null;
            Table t = _database.Tables["dbo.Person"];
            r = t.Relationships["FK_Person_Mother"];

            Assert.AreEqual(RelationshipTypeEnum.Recursive, r.RelationshipType);
            Assert.AreEqual("Person", r.PrimaryKey.TableName);
            Assert.AreEqual("Person", r.ForeignKey.TableName);

            Assert.AreEqual(1, r.ForeignKey.Columns.Count);
            Assert.AreEqual(1, r.PrimaryKey.Columns.Count);

            Assert.AreEqual("PersonID", r.PrimaryKey.Columns[0].Name);
            Assert.AreEqual("MotherID", r.ForeignKey.Columns[0].Name);

            Assert.IsNull(r.IntermediateTable);
        }

        [Test]
        public void Naming_RelationshipName()
        {
            Assert.AreEqual("CarPreviousOwners", Naming.RelationshipName(_database.Tables["dbo.Person"].Relationships["FK_Car_Person1"]));
            Assert.AreEqual("CarOwners", Naming.RelationshipName(_database.Tables["dbo.Person"].Relationships["FK_Car_Person"]));
            Assert.AreEqual("PreviousOwner", Naming.RelationshipName(_database.Tables["dbo.Car"].Relationships["FK_Car_Person1"]));
            Assert.AreEqual("Owner", Naming.RelationshipName(_database.Tables["dbo.Car"].Relationships["FK_Car_Person"]));
            Assert.AreEqual("Courses", Naming.RelationshipName(_database.Tables["dbo.Student"].Relationships["FK_StudentCourseLink_Student"]));
            Assert.AreEqual("OneToOne", Naming.RelationshipName(_database.Tables["dbo.Student"].Relationships["FK_Student_Person"]));
            Assert.AreEqual("Universities", Naming.RelationshipName(_database.Tables["dbo.Student"].Relationships["FK_University_Students_Student"]));
            Assert.AreEqual("Mother", Naming.RelationshipName(_database.Tables["dbo.Person"].Relationships["FK_Person_Mother"]));
            Assert.AreEqual("EntityTableDummy2s", Naming.RelationshipName(_database.Tables["dbo.MultipleTableNameRelationshipTest"].Relationships["FK_This_That_TheOther_EntityTable_MultipleTableNameRelationshipTest"]));
            
        }
    }
}