#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Xml;
using dnp.Framework.Database.DatabaseExplorer;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DatabaseExplorer
{
    [TestFixture]
    public class TableColumnTest
    {
        private Framework.Database.DatabaseExplorer.Database _database;

        [TestFixtureSetUp]
        public void Setup()
        {
            _database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);
        }

        [Test]
        public void Name()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual("BigInt", c.Name);

            c = t.Columns["Binary"];
            Assert.AreEqual("Binary", c.Name);

            c = t.Columns["Bit"];
            Assert.AreEqual("Bit", c.Name);

            c = t.Columns["Char"];
            Assert.AreEqual("Char", c.Name);

            c = t.Columns["DateTime"];
            Assert.AreEqual("DateTime", c.Name);

            c = t.Columns["Decimal"];
            Assert.AreEqual("Decimal", c.Name);

            c = t.Columns["Int"];
            Assert.AreEqual("Int", c.Name);

            c = t.Columns["Money"];
            Assert.AreEqual("Money", c.Name);

            c = t.Columns["Nchar"];
            Assert.AreEqual("Nchar", c.Name);

            c = t.Columns["Ntext"];
            Assert.AreEqual("Ntext", c.Name);

            c = t.Columns["Numeric"];
            Assert.AreEqual("Numeric", c.Name);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual("Nvarchar", c.Name);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual("NVarcharMax", c.Name);

            c = t.Columns["Real"];
            Assert.AreEqual("Real", c.Name);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual("SmallDateTime", c.Name);

            c = t.Columns["SmallInt"];
            Assert.AreEqual("SmallInt", c.Name);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual("SmallMoney", c.Name);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual("SqlVariant", c.Name);

            c = t.Columns["Text"];
            Assert.AreEqual("Text", c.Name);

            c = t.Columns["Timestamp"];
            Assert.AreEqual("Timestamp", c.Name);

            c = t.Columns["TinyInt"];
            Assert.AreEqual("TinyInt", c.Name);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual("UniqueIdentifier", c.Name);

            c = t.Columns["VarBinary"];
            Assert.AreEqual("VarBinary", c.Name);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual("VarBinaryMax", c.Name);

            c = t.Columns["VarChar"];
            Assert.AreEqual("VarChar", c.Name);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual("VarCharMax", c.Name);

            c = t.Columns["Xml"];
            Assert.AreEqual("Xml", c.Name);
        }

        [Test]
        public void Alias()
        {
            Table t = _database.Tables["dbo.Person"];

            // Check that aliased fields have thier alias
            Assert.AreEqual("ChristianName", t.Columns["FirstName"].Alias);
            // Check that non-aliased fields have the name of the column
            Assert.AreEqual("LastName", t.Columns["LastName"].Alias);
        }

        [Test]
        public void Description()
        {
            // Check that descriptions are correct.
            Assert.AreEqual("State issued registration number", _database.Tables["dbo.Car"].Columns["RegistrationNumber"].Description);
            // Check that non-aliased fields have the name of the column
            Assert.AreEqual("", _database.Tables["dbo.Person"].Columns["FirstName"].Description);
        }
        [Test]
        public void CharacterMaximumLength()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Binary"];
            Assert.AreEqual(50, c.CharacterMaximumLength);

            c = t.Columns["Bit"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Char"];
            Assert.AreEqual(10, c.CharacterMaximumLength);

            c = t.Columns["DateTime"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Decimal"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Int"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Money"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Nchar"];
            Assert.AreEqual(10, c.CharacterMaximumLength);

            c = t.Columns["Ntext"];
            Assert.AreEqual(1073741823, c.CharacterMaximumLength);

            c = t.Columns["Numeric"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(50, c.CharacterMaximumLength);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(-1, c.CharacterMaximumLength);

            c = t.Columns["Real"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(0, c.CharacterMaximumLength);

            c = t.Columns["Text"];
            Assert.AreEqual(2147483647, c.CharacterMaximumLength);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(null, c.CharacterMaximumLength);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(50, c.CharacterMaximumLength);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(-1, c.CharacterMaximumLength);

            c = t.Columns["VarChar"];
            Assert.AreEqual(50, c.CharacterMaximumLength);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(-1, c.CharacterMaximumLength);

            c = t.Columns["Xml"];
            Assert.AreEqual(-1, c.CharacterMaximumLength);
        }

        [Test]
        public void CollationName()
        {
            Table t = _database.Tables["dbo.Person"];

            // Check that aliased fields have thier alias
            Assert.AreEqual("SQL_Latin1_General_CP1_CI_AS", t.Columns["FirstName"].CollationName);
        }

        [Test]
        public void ColumnDefault()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Binary"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Bit"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Char"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["DateTime"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Decimal"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Int"];
            Assert.AreEqual("((10))", c.ColumnDefault);

            c = t.Columns["Money"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Nchar"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Ntext"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Numeric"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Real"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Text"];
            Assert.AreEqual("('Some Value')", c.ColumnDefault);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["VarChar"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);

            c = t.Columns["Xml"];
            Assert.AreEqual(string.Empty, c.ColumnDefault);
        }

        [Test]
        public void DataType_AllDataTypes()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual("bigint", c.DataType);

            c = t.Columns["Binary"];
            Assert.AreEqual("binary", c.DataType);

            c = t.Columns["Bit"];
            Assert.AreEqual("bit", c.DataType);

            c = t.Columns["Char"];
            Assert.AreEqual("char", c.DataType);

            c = t.Columns["DateTime"];
            Assert.AreEqual("datetime", c.DataType);

            c = t.Columns["Decimal"];
            Assert.AreEqual("decimal", c.DataType);

            c = t.Columns["Int"];
            Assert.AreEqual("int", c.DataType);

            c = t.Columns["Money"];
            Assert.AreEqual("money", c.DataType);

            c = t.Columns["Nchar"];
            Assert.AreEqual("nchar", c.DataType);

            c = t.Columns["Ntext"];
            Assert.AreEqual("ntext", c.DataType);

            c = t.Columns["Numeric"];
            Assert.AreEqual("numeric", c.DataType);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual("nvarchar", c.DataType);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual("nvarchar", c.DataType);

            c = t.Columns["Real"];
            Assert.AreEqual("real", c.DataType);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual("smalldatetime", c.DataType);

            c = t.Columns["SmallInt"];
            Assert.AreEqual("smallint", c.DataType);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual("smallmoney", c.DataType);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual("sql_variant", c.DataType);

            c = t.Columns["Text"];
            Assert.AreEqual("text", c.DataType);

            c = t.Columns["Timestamp"];
            Assert.AreEqual("timestamp", c.DataType);

            c = t.Columns["TinyInt"];
            Assert.AreEqual("tinyint", c.DataType);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual("uniqueidentifier", c.DataType);

            c = t.Columns["VarBinary"];
            Assert.AreEqual("varbinary", c.DataType);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual("varbinary", c.DataType);

            c = t.Columns["VarChar"];
            Assert.AreEqual("varchar", c.DataType);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual("varchar", c.DataType);

            c = t.Columns["Xml"];
            Assert.AreEqual("xml", c.DataType);
        }

        [Test]
        public void DataType_Dealer()
        {
            Table t = _database.Tables["test.Dealer"];
            TableColumn c;

            c = t.Columns["DealerName"];
            Assert.AreEqual("varchar", c.DataType);

            c = t.Columns["StateID"];
            Assert.AreEqual("int", c.DataType);

            c = t.Columns["Contact"];
            Assert.AreEqual("varchar", c.DataType);
        }

        /// <summary>
        /// Test that conversion from SQL Data Types to .NET Types is correct
        /// </summary>
        [Test]
        public void DotNetType_AllTypes()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(typeof(long), c.DotNetType);

            c = t.Columns["Binary"];
            Assert.AreEqual(typeof(byte[]), c.DotNetType);

            c = t.Columns["Bit"];
            Assert.AreEqual(typeof(bool), c.DotNetType);

            c = t.Columns["Char"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["DateTime"];
            Assert.AreEqual(typeof(DateTime), c.DotNetType);

            c = t.Columns["Decimal"];
            Assert.AreEqual(typeof(decimal), c.DotNetType);

            c = t.Columns["Int"];
            Assert.AreEqual(typeof(int), c.DotNetType);

            c = t.Columns["Money"];
            Assert.AreEqual(typeof(decimal), c.DotNetType);

            c = t.Columns["Nchar"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["Ntext"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["Numeric"];
            Assert.AreEqual(typeof(decimal), c.DotNetType);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["Real"];
            Assert.AreEqual(typeof(float), c.DotNetType);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(typeof(DateTime), c.DotNetType);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(typeof(short), c.DotNetType);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(typeof(decimal), c.DotNetType);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(typeof(object), c.DotNetType);

            c = t.Columns["Text"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(typeof(byte[]), c.DotNetType);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(typeof(byte), c.DotNetType);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(typeof(Guid), c.DotNetType);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(typeof(byte[]), c.DotNetType);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(typeof(byte[]), c.DotNetType);

            c = t.Columns["VarChar"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(typeof(string), c.DotNetType);

            c = t.Columns["Xml"];
            Assert.AreEqual(typeof(XmlDocument), c.DotNetType);
        }
        [Test]
        public void DotNetType_Dealer()
        {
            Table t = _database.Tables["test.Dealer"];
            TableColumn c;

            c = t.Columns["DealerName"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["StateID"];
            Assert.AreEqual(typeof (int), c.DotNetType);

            c = t.Columns["Contact"];
            Assert.AreEqual(typeof (string), c.DotNetType);
        }

        [Test]
        public void DateTimePrecision()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["DateTime"];
            Assert.AreEqual(3, c.DateTimePrecision);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(0, c.DateTimePrecision);
        }

        [Test]
        public void DotNetType()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(typeof (Int64), c.DotNetType);

            c = t.Columns["Binary"];
            Assert.AreEqual(typeof (byte[]), c.DotNetType);

            c = t.Columns["Bit"];
            Assert.AreEqual(typeof (Boolean), c.DotNetType);

            c = t.Columns["Char"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["DateTime"];
            Assert.AreEqual(typeof (DateTime), c.DotNetType);

            c = t.Columns["Decimal"];
            Assert.AreEqual(typeof (Decimal), c.DotNetType);

            c = t.Columns["Int"];
            Assert.AreEqual(typeof (Int32), c.DotNetType);

            c = t.Columns["Money"];
            Assert.AreEqual(typeof (Decimal), c.DotNetType);

            c = t.Columns["Nchar"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["Ntext"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["Numeric"];
            Assert.AreEqual(typeof (Decimal), c.DotNetType);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["Real"];
            Assert.AreEqual(typeof (Single), c.DotNetType);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(typeof (DateTime), c.DotNetType);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(typeof (Int16), c.DotNetType);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(typeof (Decimal), c.DotNetType);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(typeof (Object), c.DotNetType);

            c = t.Columns["Text"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(typeof (byte[]), c.DotNetType);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(typeof (byte), c.DotNetType);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(typeof (Guid), c.DotNetType);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(typeof (byte[]), c.DotNetType);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(typeof (byte[]), c.DotNetType);

            c = t.Columns["VarChar"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(typeof (string), c.DotNetType);

            c = t.Columns["Xml"];
            Assert.AreEqual(typeof (XmlDocument), c.DotNetType);
        }

        [Test]
        public void InPrimaryKey_Dealer()
        {
            Table t = _database.Tables["test.Dealer"];
            TableColumn c;

            c = t.Columns["DealerName"];
            Assert.IsTrue(c.InPrimaryKey);

            c = t.Columns["StateID"];
            Assert.IsTrue(c.InPrimaryKey);

            c = t.Columns["Contact"];
            Assert.IsFalse(c.InPrimaryKey);
        }

        [Test]
        public void InPrimaryKey_Region()
        {
            Table t = _database.Tables["dbo.Region"];
            TableColumn c;

            c = t.Columns["CountryID"];
            Assert.IsTrue(c.InPrimaryKey);

            c = t.Columns["StateID"];
            Assert.IsTrue(c.InPrimaryKey);

            c = t.Columns["RegionName"];
            Assert.IsFalse(c.InPrimaryKey);
        }

        [Test]
        public void InPrimaryKey_State()
        {
            Table t = _database.Tables["dbo.State"];
            TableColumn c;

            c = t.Columns["StateID"];
            Assert.IsTrue(c.InPrimaryKey);

            c = t.Columns["State"];
            Assert.IsFalse(c.InPrimaryKey);
        }

        [Test]
        public void InUniqueIndex_Car_True()
        {
            Table t = _database.Tables["dbo.Car"];
            TableColumn c;

            c = t.Columns["Make"];
            Assert.IsTrue(c.InUnquieIndex);

            c = t.Columns["Model"];
            Assert.IsTrue(c.InUnquieIndex);

            c = t.Columns["CarID"];
            Assert.IsTrue(c.InUnquieIndex);
        }

        [Test]
        public void InUniqueIndex_Car_False()
        {
            Table t = _database.Tables["dbo.Car"];
            TableColumn c;

            c = t.Columns["DealerStateID"];
            Assert.IsFalse(c.InUnquieIndex);
        }

        [Test]
        public void HasIndex()
        {
            Table t = _database.Tables["dbo.Car"];

            Assert.IsTrue(t.Columns["OwnerID"].HasIndex);
            Assert.IsTrue(t.Columns["CarID"].HasIndex);
            Assert.IsTrue(t.Columns["Make"].HasIndex);
            Assert.IsTrue(t.Columns["Model"].HasIndex);
            Assert.IsFalse(t.Columns["RegistrationNumber"].HasIndex);
        }
        [Test]
        public void IsIdentity()
        {
            Table t = _database.Tables["dbo.Car"];
            TableColumn c;

            c = t.Columns["CarID"];
            Assert.IsTrue(c.IsIdentity);

            c = t.Columns["OwnerID"];
            Assert.IsFalse(c.IsIdentity);

            c = t.Columns["Make"];
            Assert.IsFalse(c.IsIdentity);
        }

        [Test]
        public void IsNullable()
        {
            Table t = _database.Tables["dbo.Car"];
            TableColumn c;

            c = t.Columns["RegistrationNumber"];
            Assert.IsTrue(c.IsNullable);

            c = t.Columns["AverageFuelEconomy"];
            Assert.IsTrue(c.IsNullable);

            c = t.Columns["CarID"];
            Assert.IsFalse(c.IsNullable);
        }

        [Test]
        public void IsComputed()
        {
            Table t = _database.Tables["dbo.Car"];
            TableColumn c;

            c = t.Columns["RegistrationNumber"];
            Assert.IsFalse(c.IsComputed);

            c = t.Columns["AverageFuelEconomy"];
            Assert.IsTrue(c.IsComputed);
        }

        [Test]
        public void InForeignKey()
        {
            Table t = _database.Tables["dbo.Region"];
            TableColumn c;

            c = t.Columns["StateID"];
            Assert.IsTrue(c.InForeignKey);

            c = t.Columns["CountryID"];
            Assert.IsTrue(c.InForeignKey);

            c = t.Columns["RegionName"];
            Assert.IsFalse(c.InForeignKey);
        }

        [Test]
        public void NumericPrecision()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(19, c.NumericPrecision);

            c = t.Columns["Binary"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Bit"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Char"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["DateTime"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Decimal"];
            Assert.AreEqual(18, c.NumericPrecision);

            c = t.Columns["Int"];
            Assert.AreEqual(10, c.NumericPrecision);

            c = t.Columns["Money"];
            Assert.AreEqual(19, c.NumericPrecision);

            c = t.Columns["Nchar"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Ntext"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Numeric"];
            Assert.AreEqual(18, c.NumericPrecision);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Real"];
            Assert.AreEqual(24, c.NumericPrecision);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(5, c.NumericPrecision);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(10, c.NumericPrecision);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Text"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(3, c.NumericPrecision);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["VarChar"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(null, c.NumericPrecision);

            c = t.Columns["Xml"];
            Assert.AreEqual(null, c.NumericPrecision);
        }

        [Test]
        public void NumericScale()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            TableColumn c;

            c = t.Columns["BigInt"];
            Assert.AreEqual(0, c.NumericScale);

            c = t.Columns["Binary"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Bit"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Char"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["DateTime"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Decimal"];
            Assert.AreEqual(0, c.NumericScale);

            c = t.Columns["Int"];
            Assert.AreEqual(0, c.NumericScale);

            c = t.Columns["Money"];
            Assert.AreEqual(4, c.NumericScale);

            c = t.Columns["Nchar"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Ntext"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Numeric"];
            Assert.AreEqual(18, c.NumericPrecision);

            c = t.Columns["Nvarchar"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["NVarcharMax"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Real"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["SmallDateTime"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["SmallInt"];
            Assert.AreEqual(0, c.NumericScale);

            c = t.Columns["SmallMoney"];
            Assert.AreEqual(4, c.NumericScale);

            c = t.Columns["SqlVariant"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Text"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Timestamp"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["TinyInt"];
            Assert.AreEqual(0, c.NumericScale);

            c = t.Columns["UniqueIdentifier"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["VarBinary"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["VarBinaryMax"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["VarChar"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["VarCharMax"];
            Assert.AreEqual(null, c.NumericScale);

            c = t.Columns["Xml"];
            Assert.AreEqual(null, c.NumericScale);
        }

        [Test]
        public void OrdinalPosition()
        {
            Table t = _database.Tables["dbo.Person"];
            TableColumn c;

            c = t.Columns["PersonID"];
            Assert.AreEqual(1, c.OrdinalPosition);

            c = t.Columns["FirstName"];
            Assert.AreEqual(2, c.OrdinalPosition);

            c = t.Columns["LastName"];
            Assert.AreEqual(3, c.OrdinalPosition);
        }

        [Test]
        public void Parent()
        {
            Table t = _database.Tables["dbo.Student"];
            TableColumn c;

            c = t.Columns["StudentID"];
            Assert.AreEqual(t.Name, c.Parent.Name);
        }
    }
}