#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.DatabaseExplorer;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DatabaseExplorer
{
    [TestFixture]
    public class TableIndexTest
    {
        private Framework.Database.DatabaseExplorer.Database _database;

        [TestFixtureSetUp]
        public void Setup()
        {
            _database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);
        }

        [Test]
        public void IsClustered_Car_False()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["PK_Car"];

            Assert.IsFalse(index.IsClustered);
        }

        [Test]
        public void IsClustered_Person_True()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Person"];
            index = t.Indexes["PK_Person"];

            Assert.IsTrue(index.IsClustered);
        }

        [Test]
        public void IsPrimaryKey_Person_True()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Person"];
            index = t.Indexes["PK_Person"];

            Assert.IsTrue(index.IsPrimaryKey);
        }

        [Test]
        public void IsPrimaryKey_Car_False()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IXU_MakeModel"];

            Assert.IsFalse(index.IsPrimaryKey);
        }

        [Test]
        public void IsUnique_Person_True()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Person"];
            index = t.Indexes["PK_Person"];

            Assert.IsTrue(index.IsPrimaryKey);
        }

        [Test]
        public void IsUnique_Car_True()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IXU_MakeModel"];

            Assert.IsTrue(index.IsUnique);
        }

        [Test]
        public void IsUnique_Car_False()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IX_Car_OwnerID"];

            Assert.IsFalse(index.IsUnique);
        }

        [Test]
        public void Name_Car_Composite()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IXU_MakeModel"];

            Assert.AreEqual("IXU_MakeModel", index.Name);
        }

        [Test]
        public void Columns_Car_Composite()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IXU_MakeModel"];

            Assert.AreEqual("Make", index.Columns[0].Name);
            Assert.AreEqual("Model", index.Columns[1].Name);
        }

        [Test]
        public void IsDisabled_Car_True()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IX_DealerStateID"];

            Assert.IsTrue(index.IsDisabled);
        }

        [Test]
        public void IsDisabled_Car_False()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IXU_MakeModel"];

            Assert.IsFalse(index.IsDisabled);
        }

        [Test]
        public void FillFactor_Car_Default()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IX_DealerStateID"];

            Assert.AreEqual(0, index.FillFactor);
        }

        [Test]
        public void FillFactor_Car_NonDefault()
        {
            Index index = null;
            Table t = _database.Tables["dbo.Car"];
            index = t.Indexes["IX_Car_OwnerID"];

            Assert.AreEqual(50,index.FillFactor);
        }
    }
}
