#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.DatabaseExplorer;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Database.DatabaseExplorer
{
    [TestFixture]
    public class TableTest
    {
        private Framework.Database.DatabaseExplorer.Database _database;

        [TestFixtureSetUp]
        public void Setup()
        {
            _database = new Framework.Database.DatabaseExplorer.Database("ReferenceSchema", null);
        }

        [Test]
        public void Name()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];

            Assert.AreEqual("AllDataTypes", t.Name);
        }

        [Test]
        public void Alias()
        {
            // Check that aliased table names have thier alias
            Assert.AreEqual("PreviousUniversityStudent", _database.Tables["dbo.PreviousStudent"].Alias);
            // Check that non-aliased tables have the name of the table
            Assert.AreEqual("Student", _database.Tables["dbo.Student"].Alias);
        }

        [Test]
        public void FullName()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            Assert.AreEqual(t.Schema + "." + t.Name, t.FullName);
        }

        [Test]
        public void HasPrimaryKey()
        {
            Table t = _database.Tables["dbo.AllDataTypes"];
            Assert.IsFalse(t.HasPrimaryKey);

            t = _database.Tables["dbo.Student"];
            Assert.IsTrue(t.HasPrimaryKey);
        }

        [Test]
        public void Schema()
        {
            Assert.AreEqual("dbo", _database.Tables["dbo.AllDataTypes"].Schema);
            Assert.AreEqual("test", _database.Tables["test.Dealer"].Schema);
        }

        [Test]
        public void PrimaryKey_Single()
        {
            Table t = _database.Tables["dbo.Course"];
            
            Assert.IsTrue(t.HasPrimaryKey);
            Assert.AreEqual(1,t.PrimaryKey.Columns.Count);
            Assert.AreEqual(ConstraintTypeEnum.PrimaryKey,t.PrimaryKey.ConstraintType);
            Assert.AreEqual("CourseID",t.PrimaryKey.Columns[0].Name);
        }

        [Test]
        public void PrimaryKey_CompositeInteger()
        {
            Table t = _database.Tables["dbo.Region"];

            Assert.IsTrue(t.HasPrimaryKey);
            Assert.AreEqual(2, t.PrimaryKey.Columns.Count);
            Assert.AreEqual(ConstraintTypeEnum.PrimaryKey, t.PrimaryKey.ConstraintType);
            Assert.AreEqual("CountryID", t.PrimaryKey.Columns[0].Name);
            Assert.AreEqual("StateID", t.PrimaryKey.Columns[1].Name);
        }

        [Test]
        public void PrimaryKey_CompositeMixed()
        {
            Table t = _database.Tables["test.Dealer"];

            Assert.IsTrue(t.HasPrimaryKey);
            Assert.AreEqual(2, t.PrimaryKey.Columns.Count);
            Assert.AreEqual(ConstraintTypeEnum.PrimaryKey, t.PrimaryKey.ConstraintType);
            Assert.AreEqual("DealerName", t.PrimaryKey.Columns[0].Name);
            Assert.AreEqual("StateID", t.PrimaryKey.Columns[1].Name);
        }

        [Test]
        public void ForeignKey_Composite()
        {
            Table t = _database.Tables["dbo.Car"];

            Assert.IsTrue(t.Columns["DealerName"].InForeignKey);
            Assert.IsTrue(t.Columns["DealerStateID"].InForeignKey);
        }

        [Test]
        public void ForeignKey_Single()
        {
            Table t = _database.Tables["dbo.Person"];

            Assert.IsTrue(t.Columns["HomeID"].InForeignKey);
            Assert.IsFalse(t.Columns["FirstName"].InForeignKey);
        }
    }
}
