#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Scripting;
using dnp.Framework.UnitTest.Scripting.Shams;
using MbUnit.Framework;

namespace dnp.Framework.UnitTest.Scripting
{
    [TestFixture]
    public class NamingTest
    {
        [Test]
        public void PascalCase()
        {
            Assert.AreEqual("MyTest", Naming.PascalCase("myTest"));
            Assert.AreEqual("MyOtherTest", Naming.PascalCase("myOtherTest"));
            Assert.AreEqual("MyotherTest", Naming.PascalCase("MyotherTest"));
            Assert.AreEqual("MyotherTest", Naming.PascalCase("MyotherTest"));
        }

        [Test]
        public void CamelCase()
        {
            Assert.AreEqual("myTest", Naming.CamelCase("MyTest"));
            Assert.AreEqual("myOtherTest", Naming.CamelCase("MyOtherTest"));
            Assert.AreEqual("myotherTest", Naming.CamelCase("MyotherTest"));
            Assert.AreEqual("myotherTest", Naming.CamelCase("myotherTest"));
        }

        [Test]
        public void PluralName_oes_rule()
        {
            Assert.AreEqual("Heroes", Naming.PluralName("Hero"));
            Assert.AreEqual("potatoes", Naming.PluralName("potato"));
            Assert.AreEqual("volcanoes", Naming.PluralName("volcano"));
        }

        [Test]
        public void PluralName_es_rule()
        {
            Assert.AreEqual("glasses", Naming.PluralName("glass"));
            Assert.AreEqual("dishes", Naming.PluralName("dish"));
            Assert.AreEqual("judges", Naming.PluralName("judge"));
            Assert.AreEqual("phases", Naming.PluralName("phase"));
            Assert.AreEqual("witches", Naming.PluralName("witch"));
            Assert.AreEqual("houses", Naming.PluralName("house"));
            Assert.AreEqual("employees", Naming.PluralName("employee"));
        }

        [Test]
        public void PluralName_ies_rule()
        {
            Assert.AreEqual("Universities", Naming.PluralName("University"));
            Assert.AreEqual("cherries", Naming.PluralName("cherry"));
            Assert.AreEqual("ladies", Naming.PluralName("lady"));
        }

        [Test]
        public void PluralName_s_rule()
        {
            Assert.AreEqual("Aliases", Naming.PluralName("Alias"));
            Assert.AreEqual("Cathays", Naming.PluralName("Cathay"));
            Assert.AreEqual("Cars", Naming.PluralName("Car"));
            Assert.AreEqual("boys", Naming.PluralName("boy"));
            Assert.AreEqual("girls", Naming.PluralName("girl"));
            Assert.AreEqual("cats", Naming.PluralName("cat"));
            Assert.AreEqual("chairs", Naming.PluralName("chair"));
            Assert.AreEqual("mouths", Naming.PluralName("mouth"));
            Assert.AreEqual("days", Naming.PluralName("day"));
            Assert.AreEqual("monkeys", Naming.PluralName("monkey"));
            Assert.AreEqual("baths", Naming.PluralName("bath"));
        }

        [Test]
        public void PluralName_AlreadyPlural_rule()
        {
            Assert.AreEqual("Data", Naming.PluralName("Data"));
            Assert.AreEqual("cars", Naming.PluralName("cars"));
            Assert.AreEqual("heroes", Naming.PluralName("heroes"));
            Assert.AreEqual("spies", Naming.PluralName("spies"));
        }

        [Test]
        public void PluralName_ImproperNouns_rule()
        {
            Assert.AreEqual("feet", Naming.PluralName("foot"));
            Assert.AreEqual("people", Naming.PluralName("person"));
            //Assert.AreEqual("Germanys", Naming.PluralName("Germany"));
            //Assert.AreEqual("Harrys", Naming.PluralName("Harry"));
        }

        [Test]
        public void SingularName()
        {
            Assert.AreEqual("hero", Naming.SingularName("heroes"));
            Assert.AreEqual("foot", Naming.SingularName("feet"));
            Assert.AreEqual("Employee", Naming.SingularName("Employees"));
        }

        //[Test]
        //public void Pluralize_Inflector()
        //{
        //    Dictionary<string, string> _singularToPlural = new Dictionary<string, string>();
        //    _singularToPlural.Add("search", "searches");
        //    _singularToPlural.Add("switch", "switches");
        //    //_singularToPlural.Add("fix", "fixes");
        //    _singularToPlural.Add("box", "boxes");
        //    _singularToPlural.Add("process", "processes");
        //    _singularToPlural.Add("address", "addresses");
        //    _singularToPlural.Add("case", "cases");
        //    _singularToPlural.Add("stack", "stacks");
        //    _singularToPlural.Add("wish", "wishes");
        //    _singularToPlural.Add("fish", "fish");

        //    _singularToPlural.Add("category", "categories");
        //    _singularToPlural.Add("query", "queries");
        //    _singularToPlural.Add("ability", "abilities");
        //    _singularToPlural.Add("agency", "agencies");
        //    _singularToPlural.Add("movie", "movies");

        //    _singularToPlural.Add("archive", "archives");

        //    //_singularToPlural.Add("index", "indices"); // debatable

        //    _singularToPlural.Add("wife", "wives");
        //    //_singularToPlural.Add("safe", "saves");
        //    _singularToPlural.Add("half", "halves");

        //    _singularToPlural.Add("move", "moves");

        //    _singularToPlural.Add("salesperson", "salespeople");
        //    _singularToPlural.Add("person", "people");

        //    _singularToPlural.Add("spokesman", "spokesmen");
        //    _singularToPlural.Add("man", "men");
        //    _singularToPlural.Add("woman", "women");

        //    _singularToPlural.Add("basis", "bases");
        //    _singularToPlural.Add("diagnosis", "diagnoses");

        //     _singularToPlural.Add("datum", "data"); //debatable
        //    // _singularToPlural.Add("medium", "media");
        //    _singularToPlural.Add("analysis", "analyses");

        //    _singularToPlural.Add("node_child", "node_children");
        //    _singularToPlural.Add("child", "children");

        //    _singularToPlural.Add("experience", "experiences");
        //    _singularToPlural.Add("day", "days");

        //    _singularToPlural.Add("comment", "comments");
        //    _singularToPlural.Add("foobar", "foobars");
        //    _singularToPlural.Add("newsletter", "newsletters");

        //    _singularToPlural.Add("old_news", "old_news");
        //    _singularToPlural.Add("news", "news");

        //    _singularToPlural.Add("series", "series");
        //    _singularToPlural.Add("species", "species");

        //    //_singularToPlural.Add("quiz", "quizzes");

        //    _singularToPlural.Add("perspective", "perspectives");

        //    //_singularToPlural.Add("ox", "oxen"); //debatable
        //    _singularToPlural.Add("photo", "photos");
        //    _singularToPlural.Add("buffalo", "buffaloes");
        //    _singularToPlural.Add("tomato", "tomatoes");
        //    _singularToPlural.Add("dwarf", "dwarves");
        //    _singularToPlural.Add("elf", "elves");
        //    //_singularToPlural.Add("information", "information");
        //    //_singularToPlural.Add("equipment", "equipment");
        //    _singularToPlural.Add("bus", "buses");
        //    _singularToPlural.Add("status", "statuses");
        //    _singularToPlural.Add("status_code", "status_codes");
        //    _singularToPlural.Add("mouse", "mice");

        //    _singularToPlural.Add("louse", "lice");
        //    _singularToPlural.Add("house", "houses");
        //    //_singularToPlural.Add("octopus", "octopi");
        //    //_singularToPlural.Add("virus", "viri");
        //    _singularToPlural.Add("alias", "aliases");
        //    _singularToPlural.Add("portfolio", "portfolios");

        //    _singularToPlural.Add("vertex", "vertices");
        //    //_singularToPlural.Add("matrix", "matrices");

        //    _singularToPlural.Add("axis", "axes");
        //    //_singularToPlural.Add("testis", "testes");
        //    _singularToPlural.Add("crisis", "crises");

        //    //_singularToPlural.Add("rice", "rice");
        //    _singularToPlural.Add("shoe", "shoes");

        //    _singularToPlural.Add("horse", "horses");
        //    _singularToPlural.Add("prize", "prizes");
        //    _singularToPlural.Add("edge", "edges");

        //    // Now test each one
        //    foreach (KeyValuePair<string, string> keyValuePair in _singularToPlural)
        //    {
        //        Assert.AreEqual(keyValuePair.Value, Naming.PluralName(keyValuePair.Key));
        //    }
        //}
        [Test]
        public void EvaluateNamingExpression_WithChild()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$Child.Name$ $Child.FullName$";

            Assert.AreEqual("table dbo.table",
                            Naming.EvaluateNamingExpression(expression, source));
        }

        [Test]
        public void EvaluateNamingExpression()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$DomainNamespace$.Business.$EntityName$.UnitTest";

            Assert.AreEqual("dnp.Framework.Business.ExpressionTest.UnitTest",
                            Naming.EvaluateNamingExpression(expression, source));
        }

        [Test]
        public void EvaluateDotNotationExpression()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$DomainNamespace$";
            Assert.AreEqual("dnp.Framework", Naming.EvaluateDotNotationExpression(expression, source));

            expression = "$ExpressionValue.DomainNamespace$";
            Assert.AreEqual("dnp.Framework", Naming.EvaluateDotNotationExpression(expression, source));
        }

        /// <summary>
        /// This test is designed to test if the method extension syntax works. A method extension
        /// allows an expression to call a method on the value result if that object supports the 
        /// method. Ie $Name#Replace('.', ' ')$
        /// </summary>
        [Test]
        public void EvaluateNamingExpression_MethodExtensionUsingCharParameter()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$DomainNamespace#Replace('.', '/')$/Business/$EntityName#Replace('.', '/')$/UnitTest";

            Assert.AreEqual("dnp/Framework/Business/ExpressionTest/UnitTest",
                            Naming.EvaluateNamingExpression(expression, source));
        }

        [Test]
        public void EvaluateNamingExpression_MethodExtensionUsingStringParameter()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$DomainNamespace#Replace(\".\", \"/\")$/Business/$EntityName#Replace('.', '/')$/UnitTest";

            Assert.AreEqual("dnp/Framework/Business/ExpressionTest/UnitTest",
                            Naming.EvaluateNamingExpression(expression, source));
        }

        [Test]
        public void EvaluateNamingExpression_MultipleSources()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$1:DomainNamespace$.Business.$EntityName$.UnitTest - $2:FullName$";

            Assert.AreEqual("dnp.Framework.Business.ExpressionTest.UnitTest - dbo.table",
                             Naming.EvaluateNamingExpression(expression, source, source.Child));
        }

        [Test]
        public void EvaluateNamingExpression_MultipleSources_StringSource()
        {
            ExpressionValue source = new ExpressionValue();
            string expression = "$1:DomainNamespace$.Business.$EntityName$.UnitTest - $2:$";

            Assert.AreEqual("dnp.Framework.Business.ExpressionTest.UnitTest - dbo.table",
                             Naming.EvaluateNamingExpression(expression, source, source.Child.FullName));
        }
        [Test]
        public void SplitBetween()
        {
            string[] result = Naming.SplitBetween("[", "]", "Namespace.[Root].[Entity].Class");
            Assert.AreEqual(5, result.Length, "Namespace.[Root].[Entity].Class");
            Assert.AreEqual("Namespace.", result[0], "Namespace.");
            Assert.AreEqual("[Root]", result[1], "[Root]");
            Assert.AreEqual(".", result[2], ".");
            Assert.AreEqual("[Entity]", result[3], "[Entity]");
            Assert.AreEqual(".Class", result[4], ".Class");

            result = Naming.SplitBetween("$", "$", "Namespace.$Root$.$Entity$.Class");
            Assert.AreEqual(5, result.Length, "Namespace.$Root$.$Entity$.Class");
            Assert.AreEqual("Namespace.", result[0], "Namespace.");
            Assert.AreEqual("$Root$", result[1], "$Root$");
            Assert.AreEqual(".", result[2], ".");
            Assert.AreEqual("$Entity$", result[3], "$Entity$");
            Assert.AreEqual(".Class", result[4], ".Class");

            result = Naming.SplitBetween("$", "$", "$hello$$Goodbye$$Again$");
            Assert.AreEqual(3, result.Length, "$hello$$Goodbye$$Again$");
            Assert.AreEqual("$hello$", result[0], "$hello$");
            Assert.AreEqual("$Goodbye$", result[1], "$Goodbye$");
            Assert.AreEqual("$Again$", result[2], "$Again$");
        }
    }
}