#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System.Data;

namespace dnp.Framework.Database.DataProvider
{
    public abstract class DataProviderBase : IDataProvider
    {
        private string _databaseName = "";
        private string _connectionString = "";
        private string _errorMessage = "";

        public DataProviderBase()
        {
        }

        public DataProviderBase(string databaseName)
        {
            _databaseName = databaseName;
        }

        protected abstract void SetDatabaseConnection(IDbCommand cmd);

        #region IDataProvider Members

        public string DatabaseName
        {
            get { return _databaseName; }
            set { _databaseName = value; }
        }

        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        public string ErrorMessage
        {
            get { return _errorMessage; }
            set { _errorMessage = value; }
        }

        public IDataReader Execute(IDbCommand cmd)
        {
            this.SetDatabaseConnection(cmd);
            return cmd.ExecuteReader(CommandBehavior.CloseConnection);
        }

        public IDataReader Execute(string storedProcedure)
        {
            IDbCommand cmd = this.CreateCommand();
            cmd.CommandText = storedProcedure;
            cmd.CommandType = CommandType.StoredProcedure;
            this.SetDatabaseConnection(cmd);
            return this.Execute(cmd);
        }

        public object ExecuteScalar(IDbCommand cmd)
        {
            this.SetDatabaseConnection(cmd);
            object result;
            using (cmd.Connection)
            {
                result = cmd.ExecuteScalar();
            }
            return result;
        }

        public int ExecuteNonQuery(IDbCommand cmd)
        {
            this.SetDatabaseConnection(cmd);
            int recordsReturned;
            using (cmd.Connection)
            {
                recordsReturned = cmd.ExecuteNonQuery();
            }
            return recordsReturned;
        }

        #region Abstract Methods

        public abstract IDbCommand CreateCommand();

        public abstract IDataParameter CreateParameter();

        public abstract IDataParameter CreateParameter(IDbCommand command, string parameterName,
                                                       DbType dbType);

        public abstract T DeserializeObject<T>(IDbCommand command);

        #endregion

        #endregion
    }
}