#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Data;
using System.Data.SqlClient;

namespace dnp.Framework.Database.DataProvider
{
    public class SQLServerProvider : DataProviderBase
    {
        private SqlConnection _connection;

        public SQLServerProvider()
        {
        }

        public SQLServerProvider(string databaseName) : base(databaseName)
        {
        }

        protected override void SetDatabaseConnection(IDbCommand cmd)
        {
            if (_connection == null)
                _connection = new SqlConnection();

            if (_connection.State == ConnectionState.Closed)
            {
                _connection.ConnectionString = this.ConnectionString;
                _connection.Open();
            }

            cmd.Connection = _connection;
        }

        public override IDbCommand CreateCommand()
        {
            return new SqlCommand();
        }

        public override IDataParameter CreateParameter()
        {
            return new SqlParameter();
        }

        public override IDataParameter CreateParameter(IDbCommand command, string parameterName, DbType dbType)
        {
            SqlDbType sqlType;
            switch (dbType)
            {
                case DbType.AnsiString:
                    sqlType = SqlDbType.VarChar;
                    break;
                case DbType.AnsiStringFixedLength:
                    sqlType = SqlDbType.Char;
                    break;
                case DbType.Binary:
                    sqlType = SqlDbType.Binary;
                    break;
                case DbType.Boolean:
                    sqlType = SqlDbType.Bit;
                    break;
                case DbType.Byte:
                    sqlType = SqlDbType.TinyInt;
                    break;
                case DbType.Currency:
                    sqlType = SqlDbType.Money;
                    break;
                case DbType.Date:
                    sqlType = SqlDbType.DateTime;
                    break;
                case DbType.DateTime:
                    sqlType = SqlDbType.DateTime;
                    break;
                case DbType.Decimal:
                    sqlType = SqlDbType.Decimal;
                    break;
                case DbType.Double:
                    sqlType = SqlDbType.Float;
                    break;
                case DbType.Guid:
                    sqlType = SqlDbType.UniqueIdentifier;
                    break;
                case DbType.Int16:
                    sqlType = SqlDbType.SmallInt;
                    break;
                case DbType.Int32:
                    sqlType = SqlDbType.Int;
                    break;
                case DbType.Int64:
                    sqlType = SqlDbType.BigInt;
                    break;
                case DbType.Object:
                    sqlType = SqlDbType.Variant;
                    break;
                case DbType.SByte:
                    sqlType = SqlDbType.SmallInt;
                    break;
                case DbType.Single:
                    sqlType = SqlDbType.Real;
                    break;
                case DbType.String:
                    sqlType = SqlDbType.NVarChar;
                    break;
                case DbType.StringFixedLength:
                    sqlType = SqlDbType.VarChar;
                    break;
                case DbType.Time:
                    sqlType = SqlDbType.DateTime;
                    break;
                case DbType.UInt16:
                    sqlType = SqlDbType.SmallInt;
                    break;
                case DbType.UInt32:
                    sqlType = SqlDbType.Int;
                    break;
                case DbType.UInt64:
                    sqlType = SqlDbType.BigInt;
                    break;
                case DbType.VarNumeric:
                    sqlType = SqlDbType.Decimal;
                    break;
                default:
                    throw new ArgumentException("Unknown type: " + dbType);
            }
            SqlParameter objParameter = new SqlParameter(parameterName, sqlType);
            command.Parameters.Add(objParameter);
            return objParameter;
        }

        public override T DeserializeObject<T>(IDbCommand command)
        {
            IDataReader dr = this.Execute(command);
            return DataAccessExtension.DeserializeObject<T>(dr);
        }
    }
}