#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using dnp.Framework.Database.InformationSchema;

namespace dnp.Framework.Database.DatabaseExplorer
{
    public abstract class Column : SchemaBase, IColumn
    {
        private ColumnSchema _columnSchema;
        private Type _dotNetType;
        private string _sqlType = "";
        private int _size;

        public Column(object parent, Database database, ColumnSchema columnSchema)
            : base(columnSchema.ColumnName, parent, database)
        {
            _columnSchema = columnSchema;
        }


        internal ColumnSchema ColumnSchema
        {
            get { return _columnSchema; }
        }

        public bool IsIdentity
        {
            get { return _columnSchema.IsIdentity; }
        }

        /// <summary>
        /// The position number within the table. This number is 1 based.
        /// </summary>
        public int OrdinalPosition
        {
            get { return _columnSchema.OrdinalPosition; }
        }

        public string ColumnDefault
        {
            get { return _columnSchema.ColumnDefault; }
        }

        public bool IsNullable
        {
            get { return _columnSchema.IsNullable; }
        }

        public bool IsComputed
        {
            get { return _columnSchema.IsComputed; }
        }

        public string DataType
        {
            get { return _columnSchema.DataType; }
        }

        public Type DotNetType
        {
            get { return this._dotNetType; }
            internal set { this._dotNetType = value; }
        }

        /// <summary>
        /// This is the generalized size of the column. This will take into
        /// account the data type and return the data types correct size.
        /// </summary>
        public int Size
        {
            get { return _size; }
            internal set { _size = value; }
        }
        
        public string SqlType
        {
            get { return _sqlType; }
            internal set { _sqlType = value; }
        }

        public int? CharacterMaximumLength
        {
            get { return _columnSchema.CharacterMaximumLength; }
        }

        public int? NumericPrecision
        {
            get { return _columnSchema.NumericPrecision; }
        }

        public int? NumericScale
        {
            get { return _columnSchema.NumericScale; }
        }

        public int? DateTimePrecision
        {
            get
            {
                if (this.DotNetType != typeof(DateTime))
                    throw new NotSupportedException("Only the DateTime datatype supports DateTimePrecision");

                return _columnSchema.DateTimePrecision;
            }
        }

        public string CollationName
        {
            get { return _columnSchema.CollationName; }
        }

        public string SafeFullName
        {
            get { return string.Format("[{0}]", this.Name); }
        }
        #region IColumn Members

        public abstract bool InUnquieIndex
        {
            get;
        }

        public abstract bool InPrimaryKey
        {
            get;
        }

        public abstract bool InForeignKey
        {
            get;
        }

        public abstract bool HasIndex
        {
            get;
        }

        #endregion
    }
}