#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.InformationSchema;

namespace dnp.Framework.Database.DatabaseExplorer
{
    public class Constraint : SchemaBase
    {
        private ConstraintSchema _constraintSchema;
        private TableColumnCollection _columns = new TableColumnCollection();
        private ConstraintTypeEnum _constraintType;

        public Constraint()
        {
            
        }
        public Constraint(Table parent, ConstraintSchemaCollection schema):base(schema[0].ConstraintName, parent, parent.Database)
        {
            this.ConstraintSchema = schema[0];

            // Now cycle through each column schema adding it to the columns collection
            foreach (ConstraintSchema constraintSchema in schema)
            {
                ProcessConstraint(constraintSchema);
            }
        }

        public new Table Parent
        {
            get { return base.Parent as Table; }
            protected set{ base.Parent = value;}
        }
        protected ConstraintSchema ConstraintSchema
        {
            get { return _constraintSchema; }
            set { _constraintSchema = value; }
        }

        protected void ProcessConstraint(ConstraintSchema constraintSchema)
        {
            TableColumn c = this.Database.Tables[constraintSchema.TableSchema + "." + constraintSchema.TableName].Columns[constraintSchema.ColumnName];

            this.Columns.Add(c);
        }

        public string TableName
        {
            get { return this.Parent.Name; }
        }

        public string TableFullName
        {
            get { return this.TableSchema + "." + this.TableName; }
        }
        public string TableSchema
        {
            get { return this.Parent.Schema; }
        }

        public TableColumnCollection Columns
        {
            get { return _columns; }
        }

        public int Position
        {
            get { return _constraintSchema.OrdinalPosition; }
        }

        public ConstraintTypeEnum ConstraintType
        {
            get
            {
                // Based on the schema convert the string to the enum equivilant
                // then record it so we don't have to do the work more than once.
                if (_constraintType == ConstraintTypeEnum.Unknown)
                {
                    switch (_constraintSchema.ConstraintType.ToLower())
                    {
                        case "unique":
                            _constraintType = ConstraintTypeEnum.Unique;
                            break;
                        case "foreign key":
                            _constraintType = ConstraintTypeEnum.ForeignKey;
                            break;
                        case "primary key":
                            _constraintType = ConstraintTypeEnum.PrimaryKey;
                            break;
                        case "check":
                            _constraintType = ConstraintTypeEnum.Check;
                            break;
                        default:
                            _constraintType = ConstraintTypeEnum.None;
                            break;
                    }
                }
                return _constraintType;
            }
        }
    }
}