#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
namespace dnp.Framework.Database.DatabaseExplorer
{
    public abstract class SchemaBase : ISchemaBase
    {
        private string _name;
        private object _parent;
        private string _description;
        private string _alias;
        private Database _database;
        private ExtendedPropertyCollection _extendedProperties = new ExtendedPropertyCollection();

        public SchemaBase()
        {
        }

        public SchemaBase(string name, object parent, Database database)
        {
            _name = name;
            _parent = parent;
            _database = database;
        }

        public Database Database
        {
            get { return _database; }
            protected set { _database = value; }
        }

        #region ISchemaBase Members

        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public string Description
        {
            get
            {
                // The alias can be either set or derived
                if (string.IsNullOrEmpty(_description))
                {
                    // Not set so attempt to derive the value
                    if (this.ExtendedProperties.Contains("MS_Description"))
                        _description = this.ExtendedProperties["MS_Description"].Value;
                    else
                        _description = "";
                }
                return _description;
            }
            protected set { _description = value; }
        }

        public string Alias
        {
            get
            {
                // The alias can be either set or derived
                if (string.IsNullOrEmpty(_alias))
                {
                    // Not set so attempt to derive the value
                    if (this.ExtendedProperties.Contains("DE_Alias"))
                        _alias = this.ExtendedProperties["DE_Alias"].Value;
                    else
                        _alias = this.Name.Replace(" ","");
                } // if
                return _alias;
            }
            set { _alias = value; }
        }

        public ExtendedPropertyCollection ExtendedProperties
        {
            get { return this._extendedProperties; }
        }

        public virtual string FullName
        {
            get { return this.Name; }
        }

        #endregion

        public object Parent
        {
            get { return _parent; }
            protected set { _parent = value; }
        }
    }
}