#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.InformationSchema;

namespace dnp.Framework.Database.DatabaseExplorer
{
    public class Table : SchemaBase
    {
        private TableColumnCollection _Columns;
        private RelationshipCollection _Relationships = new RelationshipCollection();
        private Constraint _PrimaryKey;
        private ConstraintCollection _uniqueContraints;
        private ConstraintCollection _checkContraints;
        private IndexCollection _Indexes = new IndexCollection();
        private string _Schema;
        private ConstraintCollection _constraints = new ConstraintCollection();
        private bool _isIntersectionTable;
        private bool _isIntersectionTableCalculated;

        public Table(Database parent, ColumnSchemaCollection columns)
            : base(columns[0].TableName, parent, parent)
        {
            _Columns = new TableColumnCollection(this, columns);

            // Populate base class
            this.Schema = columns[0].TableSchema;
        }

        public new Database Parent
        {
            get { return base.Parent as Database; }
        }
        private bool IsSimpleJoin()
        {
            // Check if this table has been forced to be ignored from this calculation
            foreach (string t in this.Database.IgnoredTablesForManyToManyCalcs)
                if (t == this.Name)
                    return false;

            // Start with all the fields in the table
            int entityColumnCount = this.Columns.Count;

            // Now remove any fields that are also deemed system fields as they shouldn't be used in the calculation
            foreach (TableColumn c in this.Columns)
                if (this.Database.IsSystemField(c.Name))
                    entityColumnCount -= 1;

            if (this.HasPrimaryKey)
                return this.PrimaryKey.Columns.Count == entityColumnCount && this.Relationships.Count == 2;
            else
                return false;
        }


        public string Schema
        {
            get { return _Schema; }
            set { _Schema = value; }
        }

        public TableColumnCollection Columns
        {
            get { return _Columns; }
            internal set { _Columns = value; }
        }

        public IndexCollection Indexes
        {
            get { return _Indexes; }
            internal set { _Indexes = value; }
        }

        public ConstraintCollection Constraints
        {
            get { return this._constraints; }
            internal set { this._constraints = value; }
        }

        public Constraint PrimaryKey
        {
            get
            {
                if (_PrimaryKey == null)
                {
                    // Locate the primary key from the available constraints
                    foreach (Constraint c in this.Constraints)
                        if (c.ConstraintType == ConstraintTypeEnum.PrimaryKey)
                        {
                            _PrimaryKey = c;
                            break;
                        }
                }
                return _PrimaryKey;
            }
        }

        public ConstraintCollection UniqueConstraints
        {
            get
            {
                if (_uniqueContraints == null)
                {
                    _uniqueContraints = new ConstraintCollection();
                    // Locate the primary key from the available constraints
                    foreach (Constraint c in this.Constraints)
                        if (c.ConstraintType == ConstraintTypeEnum.Unique)
                            _uniqueContraints.Add(c);
                }
                return _uniqueContraints;
            }
        }
        public ConstraintCollection CheckConstraints
        {
            get
            {
                if (_checkContraints == null)
                {
                    _checkContraints = new ConstraintCollection();
                    // Locate the primary key from the available constraints
                    foreach (Constraint c in this.Constraints)
                        if (c.ConstraintType == ConstraintTypeEnum.Check)
                            _checkContraints.Add(c);
                }
                return _checkContraints;
            }
        }

        public RelationshipCollection Relationships
        {
            get { return _Relationships; }
            internal set { _Relationships = value; }
        }

        public override string FullName
        {
            get { return this.Schema + "." + this.Name; }
        }

        public bool IsIntersectionTable
        {
            get
            {
                if (!_isIntersectionTableCalculated)
                {
                    // See if this table is an intersection table or not
                    _isIntersectionTable = this.IsSimpleJoin();
                    _isIntersectionTableCalculated = true;
                }
                return _isIntersectionTable;
            }
        }

        public bool HasPrimaryKey
        {
            get { return this.PrimaryKey != null; }
        }

        public string SafeFullName
        {
            get { return string.Format("[{0}].[{1}]", this.Schema, this.Name); }
        }
    }
}