#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.InformationSchema;

namespace dnp.Framework.Database.DatabaseExplorer
{
    public class TableColumn : Column
    {
        private ConstraintCollection _constraints;

        public TableColumn(Table parent, ColumnSchema columnSchema)
            : base(parent, parent.Database, columnSchema)
        {
        }

        public override bool InUnquieIndex
        {
            get
            {
                foreach (Index i in this.Parent.Indexes)
                    if (i.Columns.Contains(this.Name))
                        if (i.IsUnique)
                            return true;

                return false;
            }
        }

        public new Table Parent
        {
            get { return base.Parent as Table; }
        }
        public ConstraintCollection Constraints
        {
            get
            {
                if (_constraints == null)
                {
                    _constraints = new ConstraintCollection();
                    foreach (Constraint c in this.Parent.Constraints)
                        if (c.Columns.Contains(this.Name))
                            _constraints.Add(c);
                }
                return _constraints;
            }
        }

        public override bool InPrimaryKey
        {
            get { return HasConstraintType(ConstraintTypeEnum.PrimaryKey); }
        }


        private bool HasConstraintType(ConstraintTypeEnum constraintType)
        {
            foreach (Constraint c in this.Constraints)
                if (c.ConstraintType == constraintType)
                    return true;

            return false;
        }

        public override bool InForeignKey
        {
            get { return HasConstraintType(ConstraintTypeEnum.ForeignKey); }
        }

        public override bool HasIndex
        {
            get
            {
                foreach (Index i in this.Parent.Indexes)
                    if (i.Columns.Contains(this.Name))
                        return true;

                return false;
            }
        }
    }
}