#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Data;
using System.Text;
using System.Xml;
using dnp.Framework.Database.DatabaseExplorer;
using dnp.Framework.Database.DataProvider;
using dnp.Framework.Scripting;

namespace dnp.Framework.Database.InformationSchema
{
    public abstract class InformationSchemaProviderBase : IInformationSchemaProvider
    {
        private string _connectionString = "";

        protected abstract IDataProvider DataProvider();
        protected abstract string ColumnSchemaSQL();
        protected abstract string ConstraintSchemaSQL();
        protected abstract string ForeignConstraintSchemaSQL();
        protected abstract string IndexSchemaSQL();
        protected abstract string ViewColumnSchemaSQL();
        protected abstract string ExtendedProperties();
        protected abstract string ViewTextSQL();
        protected abstract string RoutineColumnsSQL();
        protected abstract string RoutineParametersSQL();

        #region IInformationSchemaProvider Members

        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        public ColumnSchemaCollection RetrieveColumnSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.ColumnSchemaSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ColumnSchemaCollection items = new ColumnSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ColumnSchema item = new ColumnSchema();
                    item.CharacterMaximumLength = this.ConvertToNullableInt32(dr["Character_Maximum_Length"]);
                    item.CollationName = dr["Collation_Name"].ToString();
                    item.ColumnDefault = dr["Column_Default"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.DataType = dr["Data_Type"].ToString();
                    item.DateTimePrecision = this.ConvertToNullableInt32(dr["DateTime_Precision"]);
                    item.IsIdentity = this.ConvertToBool(dr["Is_Identity"]);
                    item.IsNullable = this.ConvertToBool(dr["Is_Nullable"]);
                    item.IsComputed = this.ConvertToBool(dr["Is_Computed"]);
                    item.NumericPrecision = this.ConvertToNullableInt32(dr["Numeric_Precision"]);
                    item.NumericScale = this.ConvertToNullableInt32(dr["Numeric_Scale"]);
                    item.OrdinalPosition = Convert.ToInt32(dr["Ordinal_Position"]);
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public ConstraintSchemaCollection RetrieveConstraintSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.ConstraintSchemaSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ConstraintSchemaCollection items = new ConstraintSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ConstraintSchema item = new ConstraintSchema();
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.ConstraintName = dr["Constraint_Name"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.ConstraintType = dr["Constraint_Type"].ToString();
                    item.OrdinalPosition = 1;

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public ForeignConstraintSchemaCollection RetrieveForeignConstraintSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.ForeignConstraintSchemaSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ForeignConstraintSchemaCollection items = new ForeignConstraintSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ForeignConstraintSchema item = new ForeignConstraintSchema();
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.ConstraintName = dr["Constraint_Name"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.ConstraintType = "FOREIGN KEY"; // This type of constraint is always a Foreign Key.
                    item.OrdinalPosition = Convert.ToInt32((dr["Ordinal_Position"]));


                    item.UniqueCatalog = dr["Unique_Table_Catalog"].ToString();
                    item.UniqueTableName = dr["Unique_Table_Name"].ToString();
                    item.UniqueSchema = dr["Unique_Table_Schema"].ToString();
                    item.UniqueConstraintName = dr["Unique_Constraint_Name"].ToString();
                    item.UniqueColumnName = dr["Unique_Column_Name"].ToString();

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public IndexSchemaCollection RetrieveIndexSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.IndexSchemaSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            IndexSchemaCollection items = new IndexSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    IndexSchema item = new IndexSchema();
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.ConstraintName = dr["Constraint_Name"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.OrdinalPosition = Convert.ToInt32((dr["Ordinal_Position"]));


                    item.FillFactor = Convert.ToInt32(dr["Fill_Factor"]);
                    item.IndexType = dr["Index_Type"].ToString();
                    item.IsClustered = ConvertToBool(dr["Is_Clustered"]);
                    item.IsDisabled = ConvertToBool(dr["Is_Disabled"]);

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public ColumnSchemaCollection RetrieveViewColumnSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.ViewColumnSchemaSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ColumnSchemaCollection items = new ColumnSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ColumnSchema item = new ColumnSchema();

                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.CharacterMaximumLength = this.ConvertToNullableInt32(dr["Character_Maximum_Length"]);
                    item.CollationName = dr["Collation_Name"].ToString();
                    item.ColumnDefault = dr["Column_Default"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.DataType = dr["Data_Type"].ToString();
                    item.DateTimePrecision = this.ConvertToNullableInt32(dr["DateTime_Precision"]);
                    item.IsIdentity = this.ConvertToBool(dr["Is_Identity"]);
                    item.IsNullable = this.ConvertToBool(dr["Is_Nullable"]);
                    item.IsComputed = this.ConvertToBool(dr["Is_Computed"]);
                    item.NumericPrecision = this.ConvertToNullableInt32(dr["Numeric_Precision"]);
                    item.NumericScale = this.ConvertToNullableInt32(dr["Numeric_Scale"]);
                    item.OrdinalPosition = Convert.ToInt32(dr["Ordinal_Position"]);
                    
                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public ExtendedPropertySchemaCollection RetrieveExtendedPropertySchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sql = this.ExtendedProperties();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sql;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ExtendedPropertySchemaCollection items = new ExtendedPropertySchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ExtendedPropertySchema item = new ExtendedPropertySchema();
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();

                    item.PropertyName = dr["PROPERTY_NAME"].ToString();
                    item.PropertyDescription = dr["PROPERTY_DESCRIPTION"].ToString();

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public string SQLType(ColumnSchema column)
        {
            string sqlType = "";

            switch (column.DataType)
            {
                case "char":
                case "nchar":
                case "nvarchar":
                case "varchar":
                    if (column.CharacterMaximumLength == -1)
                        sqlType = string.Format("{0} (MAX)", column.DataType);
                    else
                        sqlType = string.Format("{0} ({1})", column.DataType, column.CharacterMaximumLength);
                    break;
                case "decimal":
                case "numeric":
                    sqlType = string.Format("{0} ({1},{2})", column.DataType, column.NumericPrecision, column.NumericScale);
                    break;
                case "varbinary":
                    sqlType = string.Format("{0} (MAX)", column.DataType);
                    break;
                default:
                    sqlType = column.DataType;
                    break;
            }

            return sqlType;
        }
        public virtual Type DotNetType(string dataType)
        {
            // This conversion is based on the MSDN article
            // http://msdn2.microsoft.com/en-us/library/ms131092.aspx

            switch (dataType)
            {
                case "bigint":
                    return typeof(Int64);
                case "binary":
                    return typeof(byte[]);
                case "bit":
                    return typeof(Boolean);
                case "char":
                    return typeof(string);
                case "datetime":
                    return typeof(DateTime);
                case "decimal":
                    return typeof(Decimal);
                case "float":
                    return typeof(Double);
                case "image":
                    return typeof(byte[]);
                case "int":
                    return typeof(Int32);
                case "money":
                    return typeof(Decimal);
                case "nchar":
                    return typeof(String);
                case "ntext":
                    return typeof(String);
                case "numeric":
                    return typeof(Decimal);
                case "nvarchar":
                    return typeof(String);
                case "nvarcharmax":
                    return typeof(String);
                case "real":
                    return typeof(Single);
                case "smalldatetime":
                    return typeof(DateTime);
                case "smallint":
                    return typeof(Int16);
                case "smallmoney":
                    return typeof(Decimal);
                case "sql_variant":
                    return typeof(Object);
                case "text":
                    return typeof(String);
                case "timestamp":
                    return typeof(byte[]);
                case "tinyint":
                    return typeof(Byte);
                case "uniqueidentifier":
                    return typeof(Guid);
                case "varbinary":
                    return typeof(byte[]);
                case "varchar":
                    return typeof(String);
                case "xml":
                    return typeof(XmlDocument);

                default:
                    throw new ArgumentException("Unknown datatype: " + dataType);
            }
        }

        public virtual int Size(ColumnSchema column)
        {
            switch (column.DataType)
            {
                case "bigint":
                    return 8;
                case "binary":
                    return column.CharacterMaximumLength.Value;
                case "bit":
                    return 1;
                case "char":
                    return column.CharacterMaximumLength.Value;
                case "datetime":
                    return 8;
                case "decimal":
                    return 9;
                case "float":
                    return 8;
                case "image":
                    return 16;
                case "int":
                    return 4;
                case "money":
                    return 8;
                case "nchar":
                    return column.CharacterMaximumLength.Value * 2;
                case "ntext":
                    return 16;
                case "numeric":
                    return 9;
                case "nvarchar":
                    return column.CharacterMaximumLength.Value * 2;
                case "nvarcharmax":
                    return -1;
                case "real":
                    return 4;
                case "smalldatetime":
                    return 4;
                case "smallint":
                    return 2;
                case "smallmoney":
                    return 4;
                case "sql_variant":
                    return 8016;
                case "text":
                    return 16;
                case "timestamp":
                    return 8;
                case "tinyint":
                    return 1;
                case "uniqueidentifier":
                    return 16;
                case "varbinary":
                    return column.CharacterMaximumLength.Value;
                case "varchar":
                    return column.CharacterMaximumLength.Value;
                case "xml":
                    return -1;

                default:
                    throw new ArgumentException("Unknown datatype: " + column.DataType);
            }
        }

        public IDataReader ExecuteSQL(string sqlStatement)
        {
            //TODO: [GM] refactor code to use this general method
            IDataProvider dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlStatement;
            cmd.CommandType = CommandType.Text;

            return dataProvider.Execute(cmd);
        }

        public abstract string DatabaseName { get; }

        #endregion

        #region Helper functions

        private int? ConvertToNullableInt32(object value)
        {
            int? newValue = null;
            if (value == DBNull.Value)
                return newValue;
            else
            {
                newValue = Convert.ToInt32(value);
                return newValue;
            }
        }

        private bool ConvertToBool(object value)
        {
            switch (value.ToString().ToLower())
            {
                case "yes":
                case "true":
                case "1":
                    return true;

                case "no":
                case "false":
                case "0":
                    return false;
            }
            return false;
        }

        public string RetrieveText(SchemaBase schema)
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query, this view has embedded parameters
            string sql = Naming.EvaluateNamingExpression(this.ViewTextSQL(), schema);
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sql;
            cmd.CommandType = CommandType.Text;

            StringBuilder sqlText = new StringBuilder();
            // Define the collection to put the items into
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    sqlText.Append(dr[0]);
                } // while
            } // using

            return sqlText.ToString();
        }

        public ColumnSchemaCollection RetrieveRoutineColumnSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.RoutineColumnsSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            ColumnSchemaCollection items = new ColumnSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    ColumnSchema item = new ColumnSchema();
                    item.CharacterMaximumLength = this.ConvertToNullableInt32(dr["Character_Maximum_Length"]);
                    item.CollationName = dr["Collation_Name"].ToString();
                    item.ColumnDefault = dr["Column_Default"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.DataType = dr["Data_Type"].ToString();
                    item.DateTimePrecision = this.ConvertToNullableInt32(dr["DateTime_Precision"]);
                    item.IsIdentity = this.ConvertToBool(dr["Is_Identity"]);
                    item.IsNullable = this.ConvertToBool(dr["Is_Nullable"]);
                    item.IsComputed = this.ConvertToBool(dr["Is_Computed"]);
                    item.NumericPrecision = this.ConvertToNullableInt32(dr["Numeric_Precision"]);
                    item.NumericScale = this.ConvertToNullableInt32(dr["Numeric_Scale"]);
                    item.OrdinalPosition = Convert.ToInt32(dr["Ordinal_Position"]);
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();

                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        public RoutineParameterColumnSchemaCollection RetrieveRoutineParametersSchema()
        {
            IDataProvider dataProvider;
            dataProvider = this.DataProvider();
            dataProvider.ConnectionString = this.ConnectionString;

            // Define the SQL for the query
            string sqlColumns = this.RoutineParametersSQL();
            IDbCommand cmd = dataProvider.CreateCommand();
            cmd.CommandText = sqlColumns;
            cmd.CommandType = CommandType.Text;

            // Define the collection to put the items into
            RoutineParameterColumnSchemaCollection items = new RoutineParameterColumnSchemaCollection();
            using (IDataReader dr = dataProvider.Execute(cmd))
            {
                while (dr.Read())
                {
                    RoutineParameterColumnSchema item = new RoutineParameterColumnSchema();
                    item.TableCatalog = dr["Table_Catalog"].ToString();
                    item.TableName = dr["Table_Name"].ToString();
                    item.TableSchema = dr["Table_Schema"].ToString();
                    item.CharacterMaximumLength = this.ConvertToNullableInt32(dr["Character_Maximum_Length"]);
                    item.CollationName = dr["Collation_Name"].ToString();
                    item.ColumnDefault = dr["Column_Default"].ToString();
                    item.ColumnName = dr["Column_Name"].ToString();
                    item.DataType = dr["Data_Type"].ToString();
                    item.DateTimePrecision = this.ConvertToNullableInt32(dr["DateTime_Precision"]);
                    item.IsIdentity = false;
                    item.IsNullable = true;
                    item.IsComputed = false;
                    item.NumericPrecision = this.ConvertToNullableInt32(dr["Numeric_Precision"]);
                    item.NumericScale = this.ConvertToNullableInt32(dr["Numeric_Scale"]);
                    item.OrdinalPosition = Convert.ToInt32(dr["Ordinal_Position"]);

                    item.Direction = (RoutineParameterDirection)Enum.Parse(typeof(RoutineParameterDirection), (string)dr["PARAMETER_MODE"], true);
                    // Add to the collection
                    items.Add(item);
                } // while
            } // using

            return items;
        }

        #endregion
    }
}