#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System.Xml.Serialization;
using dnp.Framework.IO;

namespace dnp.Framework.Reflection
{
    public class Serializer
    {

        public static T Deserialize<T>(string xml)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(T));
            T objMessage;
            System.Xml.XmlTextReader reader = null;
            System.IO.StringReader sr = null;
            using (sr = new System.IO.StringReader(xml))
            {
                using (reader = new System.Xml.XmlTextReader(sr))
                {
                    objMessage = ((T)(serializer.Deserialize(reader)));
                }
            }
            return objMessage;
        }
        public static T DeserializeFromFile<T>(string fileName)
        {
            string xml = System.IO.File.ReadAllText(fileName);
            return Deserialize<T>(xml);
        }
        public static T DeserializeFromResource<T>(string resourceName)
        {
            string xml = EmbeddedResources.GetResourceString(resourceName, System.Reflection.Assembly.GetCallingAssembly());
            return Deserialize<T>(xml);
        }

        public static string Serialize<T>(T entity)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(T));
            string strXML = "";
            using (System.IO.MemoryStream writer = new System.IO.MemoryStream())
            {
                serializer.Serialize(writer, entity);
                strXML = System.Text.Encoding.UTF8.GetString(writer.GetBuffer());
            }
            return strXML;
        }
        public static void SerializeToFile<T>(T entity, string fileName)
        {
            string xml = Serialize(entity);
            System.IO.File.WriteAllText(fileName, xml);
        }
        public static string SOAPSerialize<T>(T entity)
        {
            System.Runtime.Serialization.Formatters.Soap.SoapFormatter formatter = new System.Runtime.Serialization.Formatters.Soap.SoapFormatter();
            string strXML = "";
            using (System.IO.MemoryStream writer = new System.IO.MemoryStream())
            {
                formatter.Serialize(writer, entity);
                strXML = System.Text.Encoding.UTF8.GetString(writer.GetBuffer());
            }
            return strXML;
        }
        public static byte[] BinarySerialize<T>(T entity)
        {
            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            byte[] result;
            using (System.IO.MemoryStream writer = new System.IO.MemoryStream())
            {
                formatter.Serialize(writer, entity);
                result = writer.GetBuffer();
            }
            return result;
        }
        public static T BinaryDeserialize<T>(byte[] entity)
        {
            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            using (System.IO.MemoryStream reader = new System.IO.MemoryStream(entity))
            {
                return (T)formatter.Deserialize(reader);
            }
        }
    }
}
