#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;

namespace dnp.Framework.Reflection
{
    public class Type
    {
        /// <summary>
        /// Determines if a type implements a given interface type. This will work with standard interface types as well as generic interfaces.
        /// </summary>
        /// <param name="type"></param>
        /// <param name="interfaceType"></param>
        /// <returns>returns true if type implements interfaceType otherwise returns false</returns>
        public static bool ImplementsInterface(System.Type type, System.Type interfaceType)
        {
            // Check that the interfaceType is actually an interface
            if (!interfaceType.IsInterface)
                throw new ArgumentException("interfaceType is not an interface");

            foreach (System.Type i in type.GetInterfaces())
            {
                if (i.Name == interfaceType.Name)
                    return true;
            }
            return false;
        }
        /// <summary>
        /// Determines if a class instance implements a given interface type. This will work with standard interface types as well as generic interfaces.
        /// </summary>
        /// <returns>returns true if type implements interfaceType otherwise returns false</returns>
        public static bool ImplementsInterface(object instance, System.Type interfaceType)
        {
            return ImplementsInterface(instance.GetType(), interfaceType);
        }


        public static System.Type GetPropertyReturnType(object sourceObject, string propertyName)
        {
            foreach (System.Reflection.PropertyInfo prop in sourceObject.GetType().GetProperties(System.Reflection.BindingFlags.Public))
            {
                if ((prop.Name == propertyName))
                {
                    //  We have a match
                    if ((prop.DeclaringType.ToString() == sourceObject.GetType().ToString()))
                    {
                        return prop.PropertyType;
                    }
                }
            }
            return null;
        }
        public static System.Type GetCollectionType(object sourceObject, string propertyName)
        {
            foreach (System.Reflection.PropertyInfo prop in sourceObject.GetType().GetDefaultMembers())
            {
                if ((prop.Name == propertyName))
                {
                    //  We have a match
                    if ((prop.DeclaringType.ToString() == sourceObject.GetType().ToString()))
                    {
                        return prop.PropertyType;
                    }
                }
            }
            return null;
        }
    }
}
