#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using System.Collections.Generic;
using System.Reflection;
using dnp.Framework.Database.DatabaseExplorer;

namespace dnp.Framework.Scripting
{
    public static class Naming
    {
        public static string PascalCase(string name)
        {
            return name.Substring(0, 1).ToUpper() + name.Substring(1);
        }

        public static string CamelCase(string name)
        {
            return name.Substring(0, 1).ToLower() + name.Substring(1);
        }

        #region Plural naming rules

        private static string[,] _pluralSuffixRules = new[,]
            {
                // Special Words
                {"agendum", "agenda"},
                {"albino", "albinos"},
                {"alga", "algae"},
                {"alumna", "alumnae"},
                {"apex", "apexes"},
                {"archipelago", "archipelagos"},
                {"bacterium", "bacteria"},
                {"beef", "beefs"},
                {"bison", ""},
                {"brother", "brothers"},
                {"candelabrum", "candelabra"},
                {"carp", ""},
                {"casino", "casinos"},
                {"child", "children"},
                {"chassis", ""},
                {"chinese", ""},
                {"clippers", ""},
                {"cod", ""},
                {"codex", "codices"},
                {"commando", "commandos"},
                {"corps", ""},
                {"cortex", "cortices"},
                {"cow", "cows"},
                {"criterion", "criteria"},
                //{"datum", "data"},
                {"data", ""},
                {"debris", ""},
                {"deer", "deer"},
                {"diabetes", ""},
                {"ditto", "dittos"},
                {"djinn", ""},
                {"dynamo", ""},
                {"elk", ""},
                {"embryo", "embryos"},
                {"ephemeris", "ephemeris"},
                {"erratum", "errata"},
                {"extremum", "extrema"},
                {"fiasco", "fiascos"},
                {"fish", "fish"},
                {"flounder", ""},
                {"focus", "focuses"},
                {"foot", "feet"},
                {"fungus", "fungi"},
                {"gallows", ""},
                {"genie", "genies"},
                {"ghetto", "ghettos"},
                {"goose", "geese"},
                {"graffiti", ""},
                {"headquarters", ""},
                {"herpes", ""},
                {"homework", ""},
                {"index", "indexes"},
                {"inferno", "infernos"},
                {"japanese", ""},
                {"jumbo", "jumbos"},
                {"latex", "latices"},
                {"lingo", "lingos"},
                {"mackerel", ""},
                {"macro", "macros"},
                {"man", "men"},
                {"manifesto", "manifestos"},
                {"measles", ""},
                {"money", "monies"},
                {"mongoose", "mongooses"},
                {"mumps", ""},
                {"murex", "murecis"},
                {"mythos", "mythos"},
                {"news", ""},
                {"octopus", "octopuses"},
                {"ovum", "ova"},
                {"ox", "oxes"},
                {"person", "people"},
                {"photo", "photos"},
                {"pincers", ""},
                {"pliers", ""},
                {"pro", "pros"},
                {"rabies", ""},
                {"radius", "radiuses"},
                {"rhino", "rhinos"},
                {"salmon", ""},
                {"scissors", ""},
                {"series", ""},
                {"shears", ""},
                {"sheep", "sheep"},
                {"silex", "silices"},
                {"simplex", "simplices"},
                {"soliloquy", "soliloquies"},
                {"species", ""},
                {"stratum", "strata"},
                {"swine", ""},
                {"tooth", "teeth"},
                {"trout", ""},
                {"tuna", ""},
                {"vertebra", "vertebrae"},
                {"vertex", "vertices"},
                {"vortex", "vortices"},
                {"louse", "lice"},
                {"mouse", "mice"},
                {"zoon", "zoa"},
                // Word suffix rules

                // 3 letter suffixs
                {"alf", "alves"},
                {"itis", "itis"},
                {"pox", "poxes"},
                {"elf", "elves"},
                {"ee", "ees"},
                {"olf", "olves"},
                {"arf", "arves"},
                {"leaf", "leaves"},
                {"nife", "nives"},
                {"life", "lives"},
                {"wife", "wives"},
                {"ois", "ois"},
                {"cis", "ces"},
                {"sis", "ses"},
                {"xis", "xes"},
                // 2 letter suffixs
                {"ch", "ches"},
                {"sh", "shes"},
                {"ss", "sses"},
                {"ay", "ays"},
                {"ey", "eys"},
                {"iy", "iys"},
                {"oy", "oys"},
                {"uy", "uys"},
                {"ao", "aos"},
                {"eo", "eos"},
                {"io", "ios"},
                {"oo", "oos"},
                {"ox", "oxes"},
                {"uo", "uos"},
                // 1 letter suffix
                {"y", "ies"},
                {"o", "oes"},
                {"s", "ses"}
            };

        #endregion

        public static string PluralName(string name)
        {
            string result = "";

            // Convert the name to a singluar item first to ensure we don't double up on suffixes
            name = SingularName(name);

            for (int i = 0; i <= _pluralSuffixRules.GetUpperBound(0); i++)
            {
                if (name.ToLower().EndsWith(_pluralSuffixRules[i, 0]))
                {
                    if (_pluralSuffixRules[i, 1].Length > 0)
                        result = name.Substring(0, name.Length - _pluralSuffixRules[i, 0].Length) +
                                 _pluralSuffixRules[i, 1];
                    else
                        result = name;
                    break;
                }
            }

            if (result == "")
                result = name + "s";
            return result;
        }

        public static string SingularName(string name)
        {
            string result = "";

            for (int i = 0; i <= _pluralSuffixRules.GetUpperBound(0); i++)
            {
                if (name.ToLower().EndsWith(_pluralSuffixRules[i, 1]))
                {
                    if (_pluralSuffixRules[i, 1].Length > 0)
                    {
                        result = name.Substring(0, name.Length - _pluralSuffixRules[i, 1].Length) +
                                 _pluralSuffixRules[i, 0];
                        break;
                    }
                }
            }

            if (result == "")
            {
                if (name.EndsWith("s"))
                    switch (name.Substring(name.Length - 2, 1))
                    {
                        case "a":
                        case "e":
                        case "i":
                        case "o":
                        case "u":
                        case "s":
                            // This is a natural s for this word so leave it alone
                            result = name;
                            break;
                        default:
                            // Remove the plural s from this word
                            result = name.Substring(0, name.Length - 1);
                            break;
                    }
                else
                {
                    // This is a natural s for this word so leave it alone
                    result = name;
                }
            }
            return result;
        }

        public static string PluralName2(string name)
        {
            //  This is an attempt to provide a 'fairly' accurate converstion from singular to plural
            //  Reference: http://en.wikipedia.org/wiki/English_plural
            string newName = "";
            if (name.ToLower().EndsWith("es"))
            {
                return name;
            }
            //  oes rule
            switch (name.ToLower().Substring((name.Length - 2), 1))
            {
                case "a":
                case "e":
                case "i":
                case "o":
                case "u":
                    break;
                default:
                    // consonants
                    if (name.ToLower().EndsWith("o"))
                        newName = name + "es";
                    break;
            }
            if (newName != "")
            {
                return newName;
            }
            //  ies rule
            switch (name.ToLower().Substring((name.Length - 2), 1))
            {
                case "a":
                case "e":
                case "i":
                case "o":
                case "u":
                    break;
                default:
                    if (name.ToLower().EndsWith("y"))
                        newName = name.Substring(0, name.Length - 1) + "ies";

                    break;
            }
            if (newName != "")
            {
                return newName;
            }
            //  Last letter suffix rules
            switch (name.ToLower().Substring((name.Length - 1), 1))
            {
                case "s":
                    newName = (name + "es");
                    break;
                default:
                    newName = (name + "s");
                    break;
            }
            //  Default if no change is required
            if (newName == "")
            {
                newName = name;
            }
            return newName;
        }

        /// <summary>
        /// Attempts to resolve the best name for a given column for any type of relationship.
        /// A default table expresssion returns the last word in a table name if seperated by underscores
        /// RegEx: [a-zA-Z ]+$
        /// </summary>
        /// <returns>suggested name for the relationship</returns>
        public static string RelationshipName(Relationship relationship)
        {
            return RelationshipName(relationship, @"[a-zA-Z ]+$");
        }
        /// <summary>
        /// Attempts to resolve the best name for a given column for any type of relationship.
        /// </summary>
        /// <returns>suggested name for the relationship</returns>
        public static string RelationshipName(Relationship relationship, string tableNameRegEx)
        {
            string nameSuffix = "";
            string namePrefix = "";
            string relationshipName = "";

            string leftTableAlias = relationship.Parent.Alias;
            string rightTableAlias = relationship.RightConstraint.Parent.Alias;

            if (!string.IsNullOrEmpty(tableNameRegEx))
            {
                leftTableAlias = System.Text.RegularExpressions.Regex.Matches(leftTableAlias, tableNameRegEx)[0].Value;
                rightTableAlias = System.Text.RegularExpressions.Regex.Matches(rightTableAlias, tableNameRegEx)[0].Value;
            }
            switch (relationship.RelationshipType)
            {
                case RelationshipTypeEnum.Unknown:
                    break;
                case RelationshipTypeEnum.OneToOne:
                    nameSuffix = "OneToOne";
                    break;
                case RelationshipTypeEnum.OneToMany:
                    namePrefix = NormalizeFieldName(rightTableAlias);
                    if (NormalizeFieldName(relationship.RightConstraint.Columns[0].Alias) == SingularName(leftTableAlias))
                        namePrefix = PluralName(namePrefix);
                    else
                        nameSuffix = PluralName(NormalizeFieldName(relationship.RightConstraint.Columns[0].Alias));
                    break;
                case RelationshipTypeEnum.ManyToOne:
                    namePrefix = NormalizeFieldName(relationship.LeftConstraint.Columns[0].Alias);
                    break;
                case RelationshipTypeEnum.ManyToMany:
                    nameSuffix = PluralName(NormalizeFieldName(relationship.RightConstraint.Columns[0].Alias));
                    break;
                case RelationshipTypeEnum.Recursive:
                    nameSuffix = NormalizeFieldName(relationship.LeftConstraint.Columns[0].Alias);
                    break;
                default:
                    break;
            }
            relationshipName = namePrefix + nameSuffix;
            return relationshipName;
        }

        private static string NormalizeFieldName(string fieldName)
        {
            string normalizedName;
            if (fieldName.ToLower().EndsWith("id"))
                // Remove the ID from the end of the field name
                normalizedName = fieldName.Remove(fieldName.Length - 2, 2);
            else
                normalizedName = fieldName;

            return SingularName(normalizedName);
        }
        /// <summary>
        /// Given an expression and its source evaluates the expression returing a string.
        /// </summary>
        /// <remarks>
        /// The expression handler supports the following additional features:
        ///  + suffix will return a pluralized result of the value ie $Class.Property+$
        /// </remarks>
        /// <param name="expression">A string with escaped ($...$) dot notation expressions</param>
        /// <param name="source">The root object the expression is based on.</param>
        /// <returns>string of evaluated expression</returns>
        public static string EvaluateNamingExpression(string expression, params object[] source)
        {
            if (string.IsNullOrEmpty(expression))
                return "";

            //First split the expression into parts for evaluation
            string[] expressionParts = SplitBetween("$", "$", expression);
            for (int i = 0; i < expressionParts.Length; i++)
            {
                string evaluatedPart;
                string expressionPart = expressionParts[i];
                if (expressionPart.StartsWith("$"))
                {
                    int sourceNumber = 1;
                    // Check if they've defined more than one source
                    if (expressionPart.Substring(2, 1) == ":")
                    {
                        sourceNumber = Convert.ToInt32(expressionPart.Substring(1, 1));
                        // Now strip off the source index syntax
                        expressionPart = "$" + expressionPart.Substring(3);
                    }

                    if (sourceNumber == 0)
                        throw new ArgumentException("The source index is 1 based and the supplied index is zero.");

                    if (sourceNumber > source.Length)
                        throw new ArgumentException(string.Format("The source number {0} in expression {1} is greater than the number of sources {2}", sourceNumber, expressionPart,
                                                              source.Length - 1));
                    if (expressionPart == "$$" && source[sourceNumber - 1].GetType() == typeof(string))
                        evaluatedPart = source[sourceNumber - 1].ToString();
                    else
                        evaluatedPart = EvaluateDotNotationExpression(expressionPart, source[sourceNumber - 1]);
                }
                else
                    evaluatedPart = expressionPart;

                expressionParts[i] = evaluatedPart;
            }

            string evaluatedExpression = string.Join("", expressionParts);

            return evaluatedExpression;
        }

        public static string EvaluateDotNotationExpression<T>(string expression, T source)
        {
            // Remove expression syntax
            expression = expression.Substring(1, expression.Length - 2);

            // Define the object path
            string methodExtension = "";
            if (expression.IndexOf('#') > 0)
            {
                string[] subParts = expression.Split('#');
                expression = subParts[0]; // put the expression part back to the way it was
                methodExtension = subParts[1]; // Extract the method to call
            }

            string[] paths = expression.Split('.');
            object instance = source;

            string evaluatedPath = "";
            foreach (string path in paths)
            {
                if (path == source.GetType().Name)
                {
                    evaluatedPath = path;
                    continue;
                }

                bool found = false;
                //Find the property on the instance and transfer its value to the instance variable.
                foreach (PropertyInfo p in instance.GetType().GetProperties())
                {
                    if (p.Name == path)
                    {
                        instance = p.GetValue(instance, null);
                        found = true;
                        break;
                    }
                }
                if (!found)
                    throw new ArgumentException(string.Format("Unable to evaluate the expression path {0} in expression {1} for the source {2}: ", evaluatedPath + (evaluatedPath.Length == 0 ? "" : ".") + path, expression, source.GetType().Name));

                evaluatedPath += (evaluatedPath.Length == 0 ? "" : ".") + path;

            } // foreach path
            if (methodExtension != "")
            {

                int startPos = methodExtension.IndexOf('(') + 1;
                int endPos = methodExtension.LastIndexOf(')') - startPos;
                string[] parameters = methodExtension.Substring(startPos, endPos).Split(',');
                object[] finalParams = new object[parameters.Length];

                for (int i = 0; i < parameters.Length; i++)
                    finalParams[i] = ProcessParameter(parameters[i]);

                MethodInfo method = GetMethodForParameters(instance, evaluatedPath, methodExtension, finalParams);
                return method.Invoke(instance, finalParams).ToString();
            }
            else
                // Return the instance value that has no extension method.
                return instance.ToString();
        }

        private static MethodInfo GetMethodForParameters(object instance, string evaluatedPath, string methodExtension, object[] parameters)
        {
            // Seems the user wants to execute a method on the active instance too.
            MethodInfo method = null;
            string methodName = methodExtension.Substring(0, methodExtension.IndexOf('('));
            foreach (MethodInfo m in instance.GetType().GetMethods())
                if (m.Name == methodName && m.GetParameters().Length == parameters.Length)
                {
                    bool found = false;
                    // Now compare the paramter types to match the parameters given
                    for (int i = 0; i < parameters.Length; i++)
                    {
                        if (parameters[i].GetType().Name == m.GetParameters()[0].ParameterType.Name)
                            found = true;
                        else
                        {
                            found = false;
                            break;
                        }
                    }

                    if (found)
                    {
                        method = m;
                        break;
                    }
                }

            if (method == null)
                throw new ArgumentException(string.Format("Unable to evaluation method {0} on expression path {1}", methodName, evaluatedPath));
            return method;
        }

        /// <summary>
        /// This function is used to convert a string paramter into its correct data type and return as a object
        /// </summary>
        /// <param name="paramterValue">the paramter including any surounding quotes. This is necessary to correctly identify the data type.</param>
        /// <returns></returns>
        private static object ProcessParameter(string paramterValue)
        {
            object value = null;
            paramterValue = paramterValue.Trim(); // Remove any whitespace
            string rawValue = paramterValue.Substring(1, paramterValue.Length - 2).Trim();
            if (paramterValue.StartsWith("\"")) // string type
                value = rawValue;
            else if (paramterValue.StartsWith("'")) // Char type
            {
                if (rawValue.Length > 1)
                    throw new ArgumentException("The value: {0} has been defined as a char, but is longer than one character. You need to enclose strings in quotes rather than single quotes.");

                value = rawValue.ToCharArray()[0];
            }
            else // value must be numeric
                if (paramterValue.IndexOf(".") > 0)
                    value = Convert.ToDecimal(rawValue);
                else
                    value = Convert.ToInt32(rawValue);

            return value;
        }

        /// <summary>
        /// Returns an array of strings broken up by the start and end character. A different character could be used
        /// where you needed to seperate by [ and ] for example. The result also includes the start and end characters.
        /// </summary>
        /// <param name="startCharacter">Character that denotes the start of a group</param>
        /// <param name="endCharacter">Character that denotes the end of a group</param>
        /// <param name="source">The string to be split</param>
        /// <returns></returns>
        public static string[] SplitBetween(string startCharacter, string endCharacter, string source)
        {
            List<string> result = new List<string>();
            string part = "";
            bool startFound = false;

            for (int i = 0; i < source.Length; i++)
            {
                string character = source[i].ToString();

                if (character == startCharacter && !startFound)
                {
                    startFound = true;
                    if (part != "")
                        result.Add(part);

                    // Start the new part with the start character
                    part = character;
                }
                else if (character == endCharacter && startFound)
                {
                    startFound = false;
                    // Include the end character as part of this part
                    part += character;
                    result.Add(part);
                    // Next part starts with blank
                    part = "";
                }
                else
                {
                    part += character;
                }
            }
            // Add last result
            if (part != "")
                result.Add(part);

            return result.ToArray();
        }

        public static string ProperCase(string value)
        {
            string[] words = value.Split(' ');
            TextList newWords = new TextList();
            foreach (string w in words)
                if (w.Length > 1)
                    newWords.Add(w.Substring(0, 1).ToUpper() + w.Substring(1));
                else
                    newWords.Add(w.ToUpper());

            return newWords.ToString(" ");
        }
    }
}