#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.ServiceModel.Configuration;
using dnp.UnitTestEx;
using MbUnit.Framework;

namespace dnp.ServiceModel.Test
{
    [TestFixture]
    public class BusinessServicesTest
    {
        private BusinessLayer _businessLayer;

        [TestFixtureSetUp]
        public void InitializeTest()
        {
            ServiceModelDefinition expectedModel = Testing.IO.Serializer.RetrieveResourceObject<ServiceModelDefinition>("dnpModelConfiguration.xml");

            _businessLayer = new dnpModel(expectedModel).BusinessLayer;
        }

        [Test]
        public void Name()
        {
            Assert.AreEqual("AllDataTypesService", _businessLayer.BusinessServices[0].Name);
            Assert.AreEqual("StudentService", _businessLayer.BusinessServices["StudentService"].Name);
        }

        [Test]
        public void FileName()
        {
            Assert.AreEqual(@"c:\temp\modeltest\ReferenceSchema\Business.Services\\AllDataTypesService", _businessLayer.BusinessServices[0].Filename);
        }

        [Test]
        public void GeneratedFilename()
        {
            Assert.AreEqual(@"c:\temp\modeltest\ReferenceSchema\Business.Services\Generated\\AllDataTypesService", _businessLayer.BusinessServices[0].GeneratedFilename);
        }

        [Test]
        public void InterfaceName()
        {
            Assert.AreEqual(@"IAllDataTypesService", _businessLayer.BusinessServices[0].InterfaceName);
        }

        [Test]
        public void InterfaceFilename()
        {
            Assert.AreEqual(@"c:\temp\modeltest\ReferenceSchema\Business.Services.Interfaces\\IAllDataTypesService", _businessLayer.BusinessServices[0].InterfaceFilename);
        }
        [Test]
        public void InterfaceGeneratedFilename()
        {
            Assert.AreEqual(@"c:\temp\modeltest\ReferenceSchema\Business.Services.Interfaces\Generated\\IAllDataTypesService", _businessLayer.BusinessServices[0].InterfaceGeneratedFilename);
        }
        [Test]
        public void InterfaceRelativeNamespace()
        {
            Assert.AreEqual(@"Business.Services.Interfaces", _businessLayer.BusinessServices[0].InterfaceRelativeNamespace);
        }
        [Test]
        public void InterfaceNamespace()
        {
            Assert.AreEqual(@"DotNetProfessional.ReferenceSchema.Business.Services.Interfaces", _businessLayer.BusinessServices[0].InterfaceNamespace);
        }        
        [Test]
        public void Parent()
        {
            Assert.IsNotNull(_businessLayer.BusinessServices[5].Parent);
        }

        [Test]
        public void Methods()
        {
            Assert.IsNotNull(_businessLayer.BusinessServices[0].Methods);
            Assert.AreEqual(0, _businessLayer.BusinessServices["AllDataTypesService"].Methods.Count);
            Assert.AreEqual(11, _businessLayer.BusinessServices["StudentService"].Methods.Count);
        }

        [Test]
        public void ServiceMethods()
        {
            ServiceModelDefinition expectedModel = Testing.IO.Serializer.RetrieveResourceObject<ServiceModelDefinition>("dnpModelConfiguration.xml");

            _businessLayer = new dnpModel(expectedModel).BusinessLayer;

            foreach (BusinessService bs in _businessLayer.BusinessServices)
            {
                System.Diagnostics.Debug.WriteLine(string.Format("{0}", bs.Name));
                foreach (ServiceMethod sm in bs.Methods)
                    System.Diagnostics.Debug.WriteLine(string.Format("\t {0} ({1})", sm.Name, sm.StoredProcedureName));
            }
        }
    }
}
