#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.Framework.Database.DatabaseExplorer;
using dnp.ServiceModel.Configuration;
using dnp.UnitTestEx;
using MbUnit.Framework;

namespace dnp.ServiceModel.Test
{
    [TestFixture]
    public class ServiceMethodTest
    {
        private BusinessLayer _businessLayer;

        [TestFixtureSetUp]
        public void InitializeTest()
        {
            ServiceModelDefinition expectedModel = Testing.IO.Serializer.RetrieveResourceObject<ServiceModelDefinition>("dnpModelConfiguration.xml");

            _businessLayer = new dnpModel(expectedModel).BusinessLayer;
        }

        [Test]
        [Ignore("To be provided.")]
        public void SqlRole()
        {

        }

        #region Name
        [Test]
        [Ignore("Not implemented yet.")]
        public void Name_RetrieveSingle()
        {
            Assert.AreEqual("RetrieveStudent", _businessLayer.BusinessServices["StudentService"].Methods["RetrieveStudent"].Name);
            Assert.AreEqual("RetrieveUniversity", _businessLayer.BusinessServices["UniversityService"].Methods["RetrieveUniversity"].Name);
            Assert.AreEqual("RetrieveAllDataTypes", _businessLayer.BusinessServices["AllDataTypesService"].Methods["RetrieveAllDataTypes"].Name);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void Name_RetrieveMultiple()
        {
            Assert.AreEqual("StudentSearch", _businessLayer.BusinessServices["StudentService"].Methods["StudentSearch"].Name);
            Assert.AreEqual("UniversitySearch", _businessLayer.BusinessServices["UniversityService"].Methods["UniversitySearch"].Name);
            Assert.AreEqual("AllDataTypesSearch", _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesSearch"].Name);
        }
        [Test]
        public void Name_Store()
        {
            Assert.AreEqual("StudentStore", _businessLayer.BusinessServices["StudentService"].Methods["StudentStore"].Name);
            Assert.AreEqual("UniversityStore", _businessLayer.BusinessServices["UniversityService"].Methods["UniversityStore"].Name);
            Assert.AreEqual(0, _businessLayer.BusinessServices["AllDataTypesService"].Methods.Count);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void Name_Delete()
        {
            Assert.AreEqual("StudentDelete", _businessLayer.BusinessServices["StudentService"].Methods["StudentDelete"].Name);
            Assert.AreEqual("UniversityDelete", _businessLayer.BusinessServices["UniversityService"].Methods["UniversityDelete"].Name);
            Assert.AreEqual("AllDataTypesDelete", _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesDelete"].Name);
        }

        #endregion

        #region StoredProcedure
        [Test]
        [Ignore("Not implemented yet.")]
        public void StoredProcedure_RetrieveSingle()
        {
            Assert.AreEqual("gsp_RetrieveStudent", _businessLayer.BusinessServices["StudentService"].Methods["RetrieveStudent"].StoredProcedureName);
            Assert.AreEqual("gsp_RetrieveUniversity", _businessLayer.BusinessServices["UniversityService"].Methods["RetrieveUniversity"].StoredProcedureName);
            Assert.AreEqual("gsp_RetrieveAllDataTypes", _businessLayer.BusinessServices["AllDataTypesService"].Methods["RetrieveAllDataTypes"].StoredProcedureName);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void StoredProcedure_RetrieveMultiple()
        {
            Assert.AreEqual("gsp_StudentSearch", _businessLayer.BusinessServices["StudentService"].Methods["StudentSearch"].StoredProcedureName);
            Assert.AreEqual("gsp_UniversitySearch", _businessLayer.BusinessServices["UniversityService"].Methods["UniversitySearch"].StoredProcedureName);
            Assert.AreEqual("gsp_AllDataTypesSearch", _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesSearch"].StoredProcedureName);
        }
        [Test]
        public void StoredProcedure_Store()
        {
            Assert.AreEqual("gsp_StudentStore", _businessLayer.BusinessServices["StudentService"].Methods["StudentStore"].StoredProcedureName);
            Assert.AreEqual("gsp_UniversityStore", _businessLayer.BusinessServices["UniversityService"].Methods["UniversityStore"].StoredProcedureName);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void StoredProcedure_Delete()
        {
            Assert.AreEqual("gsp_StudentDelete", _businessLayer.BusinessServices["StudentService"].Methods["StudentDelete"].StoredProcedureName);
            Assert.AreEqual("gsp_UniversityDelete", _businessLayer.BusinessServices["UniversityService"].Methods["UniversityDelete"].StoredProcedureName);
            Assert.AreEqual("gsp_AllDataTypesDelete", _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesDelete"].StoredProcedureName);
        }

        #endregion

        #region ServiceFeature
        [Test]
        [Ignore("Not implemented yet.")]
        public void ServiceType_RetrieveSingle()
        {
            Assert.AreEqual(ServiceFeatureEnum.RetrieveSingle, _businessLayer.BusinessServices["StudentService"].Methods["RetrieveStudent"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.RetrieveSingle, _businessLayer.BusinessServices["UniversityService"].Methods["RetrieveUniversity"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.RetrieveSingle, _businessLayer.BusinessServices["AllDataTypesService"].Methods["RetrieveAllDataTypes"].ServiceFeature);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void ServiceType_RetrieveMultiple()
        {
            Assert.AreEqual(ServiceFeatureEnum.RetrieveMultiple, _businessLayer.BusinessServices["StudentService"].Methods["StudentSearch"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.RetrieveMultiple, _businessLayer.BusinessServices["UniversityService"].Methods["UniversitySearch"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.RetrieveMultiple, _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesSearch"].ServiceFeature);
        }
        [Test]
        public void ServiceType_Store()
        {
            Assert.AreEqual(ServiceFeatureEnum.StoreSingle, _businessLayer.BusinessServices["StudentService"].Methods["StudentStore"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.StoreSingle, _businessLayer.BusinessServices["UniversityService"].Methods["UniversityStore"].ServiceFeature);
            Assert.AreEqual(0, _businessLayer.BusinessServices["AllDataTypesService"].Methods.Count);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void ServiceType_Delete()
        {
            Assert.AreEqual(ServiceFeatureEnum.DeleteSingle, _businessLayer.BusinessServices["StudentService"].Methods["StudentDelete"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.DeleteSingle, _businessLayer.BusinessServices["UniversityService"].Methods["UniversityDelete"].ServiceFeature);
            Assert.AreEqual(ServiceFeatureEnum.DeleteSingle, _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesDelete"].ServiceFeature);
        }

        #endregion

        #region IsCustomImplementation

        [Test]
        [Ignore("Not implemented yet.")]
        public void RequestDefinitionType_RetrieveSingle()
        {
            Assert.AreEqual(false, _businessLayer.BusinessServices["StudentService"].Methods["RetrieveStudent"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["UniversityService"].Methods["RetrieveUniversity"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["AllDataTypesService"].Methods["RetrieveAllDataTypes"].IsCustomImplementation);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void RequestDefinitionType_RetrieveMultiple()
        {
            Assert.AreEqual(false, _businessLayer.BusinessServices["StudentService"].Methods["StudentSearch"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["UniversityService"].Methods["UniversitySearch"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesSearch"].IsCustomImplementation);
        }
        [Test]
        public void RequestDefinitionType_Store()
        {
            Assert.AreEqual(false, _businessLayer.BusinessServices["StudentService"].Methods["StudentStore"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["UniversityService"].Methods["UniversityStore"].IsCustomImplementation);
        }
        [Test]
        [Ignore("Not implemented yet.")]
        public void RequestDefinitionType_Delete()
        {
            Assert.AreEqual(false, _businessLayer.BusinessServices["StudentService"].Methods["StudentDelete"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["UniversityService"].Methods["UniversityDelete"].IsCustomImplementation);
            Assert.AreEqual(false, _businessLayer.BusinessServices["AllDataTypesService"].Methods["AllDataTypesDelete"].IsCustomImplementation);
        }

        #endregion

        #region .   Exposed Entity - Store.Single

        [Test]
        public void ExposedEntity_Student_StoreSingle()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["StudentService"].Methods["StudentStore"];
            Assert.IsFalse(serviceMethod.ReturnType.IsEnumerable);

            Assert.AreEqual(false, serviceMethod.IsCustomImplementation);
            Assert.AreEqual("StudentStore", serviceMethod.Name);
            Assert.AreEqual("dl.Student", serviceMethod.ReturnType.FullName);
            Assert.AreEqual(false, serviceMethod.ReturnType.IsEnumerable);
            Assert.AreEqual(true, serviceMethod.ReturnType.IsKnownType);
            Assert.AreEqual("dl", serviceMethod.ReturnType.Namespace);
            Assert.AreEqual("Student", serviceMethod.ReturnType.Type);
            Assert.AreEqual(ServiceFeatureEnum.StoreSingle, serviceMethod.ServiceFeature);
            Assert.AreEqual("gsp_StudentStore", serviceMethod.StoredProcedureName);

            #region Parameters
            // Now Process the Parameters
            Assert.AreEqual(1, serviceMethod.Parameters.Count);
            Assert.AreEqual("entity", serviceMethod.Parameters[0].Name);
            Assert.AreEqual("Student", serviceMethod.Parameters[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[0].RelativeNamespace);
            Assert.AreEqual("dl.Student", serviceMethod.Parameters[0].RelativeName);
            #endregion

            #region Parameter Mappings
            // Now Process Parameter mappings
            int index = 0;
            Assert.AreEqual(7, serviceMethod.StoredProcedureMapping.Count);
            Assert.AreEqual("StudentId", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.StudentId", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);  // TODO: [GM] Need to determine business rule on out parm for derived classes.
            index++;

            Assert.AreEqual("GPA", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.GPA", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Decimal", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("ChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("LastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("HomeAddressAddress", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.HomeAddress.Address", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("MotherChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Mother.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("MotherLastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Mother.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            #endregion

            #region Related Methods

            // Test related methods
            // Expected Relationships: (method count)
            // Derived - UniversityStudent (1)
            // 1:M - None                  (0)
            // M:1 - None                  (0)
            // 1:1 - Person                (1)
            // M:M - Courses, Universities (2)
            // Total                       (4)

            Assert.AreEqual(4, serviceMethod.RelatedMethods.Count);

            #region Derived Definitions
            index = 0;
            // Derived
            Assert.AreEqual("PreviousUniversityStudentStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.PreviousUniversityStudentService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("PreviousUniversityStudent", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.PreviousUniversityStudent", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.Derived, serviceMethod.RelatedMethods[index].RelationshipType);
            #endregion

            #region 1:1 Definitions
            index++;
            Assert.AreEqual("PersonStoreRelationships", serviceMethod.RelatedMethods[index].MethodName); // We want to execute the base entities relationships only
            Assert.AreEqual("sl.PersonService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.OneToOne, serviceMethod.RelatedMethods[index].RelationshipType);
            #endregion

            #region M:M Definitions
            index++;
            Assert.AreEqual("StudentCoursesStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.StudentService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(2, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            //Param 1
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("PersonBusinessKey", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.PersonBusinessKey", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            //Param 2
            Assert.AreEqual("entity.Courses", serviceMethod.RelatedMethods[index].ParameterMapping[1].Name);
            Assert.AreEqual("CourseCollection", serviceMethod.RelatedMethods[index].ParameterMapping[1].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[1].RelativeNamespace);
            Assert.AreEqual("dl.CourseCollection", serviceMethod.RelatedMethods[index].ParameterMapping[1].RelativeName);

            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, serviceMethod.RelatedMethods[index].RelationshipType);

            index++;
            Assert.AreEqual("StudentUniversitiesStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.StudentService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(2, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            //Param 1
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("PersonBusinessKey", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.PersonBusinessKey", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            //Param 2
            Assert.AreEqual("entity.Universities", serviceMethod.RelatedMethods[index].ParameterMapping[1].Name);
            Assert.AreEqual("UniversityCollection", serviceMethod.RelatedMethods[index].ParameterMapping[1].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[1].RelativeNamespace);
            Assert.AreEqual("dl.UniversityCollection", serviceMethod.RelatedMethods[index].ParameterMapping[1].RelativeName);

            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, serviceMethod.RelatedMethods[index].RelationshipType);
            #endregion

            
            #endregion
        }

        [Test]
        public void ExposedEntity_Student_CourseStoreSingle()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["StudentService"].Methods["StudentCourseStore"];
            Assert.IsFalse(serviceMethod.ReturnType.IsEnumerable);

            Assert.AreEqual(false, serviceMethod.IsCustomImplementation);
            Assert.AreEqual("StudentCourseStore", serviceMethod.Name);
            Assert.AreEqual(false, serviceMethod.ReturnType.IsEnumerable);
            Assert.AreEqual(true, serviceMethod.ReturnType.IsKnownType);
            Assert.AreEqual("dl", serviceMethod.ReturnType.Namespace);
            Assert.AreEqual("Course", serviceMethod.ReturnType.Type);
            Assert.AreEqual("dl.Course", serviceMethod.ReturnType.FullName);
            Assert.AreEqual(ServiceFeatureEnum.StoreManyToMany, serviceMethod.ServiceFeature);
            Assert.AreEqual("gsp_StudentCourseStore", serviceMethod.StoredProcedureName);

            #region Parameters
            // Now Process the Parameters
            Assert.AreEqual(2, serviceMethod.Parameters.Count);
            //Param 1
            Assert.AreEqual("parent", serviceMethod.Parameters[0].Name);
            Assert.AreEqual("PersonBusinessKey", serviceMethod.Parameters[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[0].RelativeNamespace);
            Assert.AreEqual("dl.PersonBusinessKey", serviceMethod.Parameters[0].RelativeName);

            //Param 2
            Assert.AreEqual("entity", serviceMethod.Parameters[1].Name);
            Assert.AreEqual("Course", serviceMethod.Parameters[1].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[1].RelativeNamespace);
            Assert.AreEqual("dl.Course", serviceMethod.Parameters[1].RelativeName);
            #endregion

            #region Parameter Mappings
            // Now Process Parameter mappings
            Assert.AreEqual(5, serviceMethod.StoredProcedureMapping.Count);

            // Parent key definition
            int index = 0;
            Assert.AreEqual("ParentChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("parent.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("ParentLastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("parent.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            // Main properties of related entity
            index++;
            Assert.AreEqual("CourseId", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.CourseId", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(true, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("UniversityUniversityName", serviceMethod.StoredProcedureMapping[index].ParameterName); // Strange Name as its a business key
            Assert.AreEqual("entity.University.UniversityName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("Name", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Name", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            #endregion

            #region Related Methods

            // Test related methods
            // Expected Relationships: (method count)
            // Derived - None           (0)
            // 1:M - Course             (0)
            // M:1 - None               (0)
            // 1:1 - None               (0)
            // M:M - None               (1)
            // Total                    (1)

            Assert.AreEqual(1, serviceMethod.RelatedMethods.Count);

            #region M:M Definitions (These are handled differently for M:M methods)
            index = 0;
            Assert.AreEqual("CourseStoreRelationships", serviceMethod.RelatedMethods[index].MethodName); // We want to execute the related entities relationships only
            Assert.AreEqual("sl.CourseService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Course", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Course", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, serviceMethod.RelatedMethods[index].RelationshipType);

            #endregion

            #endregion
        }

        [Test]
        // The main case has been tested via Student - this case is for computed values.
        public void ExposedEntity_Car_StoreSingle()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["CarService"].Methods["CarStore"];
            Assert.IsFalse(serviceMethod.ReturnType.IsEnumerable);

            // Now Process the Parameters
            Assert.AreEqual(1, serviceMethod.Parameters.Count);
            Assert.AreEqual("entity", serviceMethod.Parameters[0].Name);
            Assert.AreEqual("Car", serviceMethod.Parameters[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[0].RelativeNamespace);

            #region Parameter Mappings

            // Now Process Parameter mappings
            int index = 0;
            Assert.AreEqual(12, serviceMethod.StoredProcedureMapping.Count);
            Assert.AreEqual("CarId", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.CarId", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(true, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("Make", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Make", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("Model", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Model", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("OwnerChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Owner.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("OwnerLastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Owner.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("PreviousOwnerChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.PreviousOwner.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("PreviousOwnerLastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.PreviousOwner.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("RegistrationNumber", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.RegistrationNumber", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("MinFuelEconomy", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.MinFuelEconomy", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("MaxFuelEconomy", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.MaxFuelEconomy", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("DealerName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.DealerName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            Assert.AreEqual("DealerStateId", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.DealerStateId", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("Int32", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);
            index++;

            #endregion

            #region Related Methods

            // Test related methods
            // Expected Relationships: (method count)
            // Derived - None                       (0)
            // 1:M - None                           (0)
            // M:1 - Owner, PreviousOwner           (2)
            // 1:1 - None                           (0)
            // M:M - None                           (0)
            // Total                                (3)

            Assert.AreEqual(2, serviceMethod.RelatedMethods.Count);

            #region M:1 Definitions
            index = 0;
            Assert.AreEqual("PersonStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.PersonService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity.Owner", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, serviceMethod.RelatedMethods[index].RelationshipType);

            index++;
            Assert.AreEqual("PersonStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.PersonService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity.PreviousOwner", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Person", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, serviceMethod.RelatedMethods[index].RelationshipType);

            #endregion

            #endregion

        }

        [Test]
        // Used to test relationship cases
        public void ExposedEntity_Person_StoreSingle()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["PersonService"].Methods["PersonStore"];

            // Test 1:M relationships
            int index = 0;
            //TODO: [GM] Add this test back in once model is more stable.
            //Assert.AreEqual(6, serviceMethod.RelatedMethods.Count);
            index = 2;
            Assert.AreEqual("CarsStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.CarService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity.CarOwners", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("CarCollection", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.CarCollection", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.OneToMany, serviceMethod.RelatedMethods[index].RelationshipType);

        }

        #endregion

        #region .   Exposed Entity - Store.Multiple

        [Test]
        // Used to test relationship cases
        public void ExposedEntity_Student_StoreMultiple()
        {

            ServiceMethod serviceMethod = _businessLayer.BusinessServices["StudentService"].Methods["StudentsStore"];

            Assert.IsTrue(serviceMethod.ReturnType.IsEnumerable);
            int index = 0;
            Assert.AreEqual("StudentStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.StudentService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Student", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Student", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.ManyToOne, serviceMethod.RelatedMethods[index].RelationshipType);
        }

        [Test]
        public void ExposedEntity_Car_StoreMulitple()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["CarService"].Methods["CarsStore"];
            Assert.IsTrue(serviceMethod.ReturnType.IsEnumerable);

            // Now Process the Parameters
            Assert.AreEqual(1, serviceMethod.Parameters.Count);
            Assert.AreEqual("entities", serviceMethod.Parameters[0].Name);
            Assert.AreEqual("CarCollection", serviceMethod.Parameters[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[0].RelativeNamespace);

        }

        #endregion

        #region .   Exposed Entity - Store.ManyToMany

        [Test]
        // Used to test relationship cases
        public void ExposedEntity_StudentCourse_ManyToManyStore()
        {

            ServiceMethod serviceMethod = _businessLayer.BusinessServices["StudentService"].Methods["StudentCourseStore"];

            Assert.IsFalse(serviceMethod.ReturnType.IsEnumerable);
            Assert.AreEqual("Course", serviceMethod.ReturnType.KnownType.Name);

            // Now Process the Parameters
            Assert.AreEqual(2, serviceMethod.Parameters.Count);
            Assert.AreEqual("parent", serviceMethod.Parameters[0].Name);
            Assert.AreEqual("PersonBusinessKey", serviceMethod.Parameters[0].DotNetType); // The PersonBusinessKey may not look correct, however we can't use StudentBusinenssKey as it doesnt exist.
            Assert.AreEqual("dl", serviceMethod.Parameters[0].RelativeNamespace);
            Assert.AreEqual("dl.PersonBusinessKey", serviceMethod.Parameters[0].RelativeName);

            Assert.AreEqual("entity", serviceMethod.Parameters[1].Name);
            Assert.AreEqual("CourseBusinessKey", serviceMethod.Parameters[1].DotNetType);
            Assert.AreEqual("dl", serviceMethod.Parameters[1].RelativeNamespace);
            Assert.AreEqual("dl.CourseBusinessKey", serviceMethod.Parameters[1].RelativeName);

            #region Parameter Mappings

            // Now Process Parameter mappings
            Assert.AreEqual(3, serviceMethod.StoredProcedureMapping.Count);

            int index = 0;
            Assert.AreEqual("ParentChristianName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("parent.ChristianName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("ParentLastName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("parent.LastName", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            index++;
            Assert.AreEqual("CourseName", serviceMethod.StoredProcedureMapping[index].ParameterName);
            Assert.AreEqual("entity.Name", serviceMethod.StoredProcedureMapping[index].PropertyMapping);
            Assert.AreEqual("String", serviceMethod.StoredProcedureMapping[index].DbType);
            Assert.AreEqual(false, serviceMethod.StoredProcedureMapping[index].IsOutput);

            #endregion

            #region Related Methods

            // Test related methods
            // Expected Relationships: (method count)
            // Derived - None                       (0)
            // 1:M -                                (0)
            // M:1 - None                           (0)
            // 1:1 - None                           (0)
            // M:M - Relationships                  (1)
            // Total                                (1)

            Assert.AreEqual(1, serviceMethod.RelatedMethods.Count);

            index = 0;
            Assert.AreEqual("CourseStore", serviceMethod.RelatedMethods[index].MethodName);
            Assert.AreEqual("sl.CourseService", serviceMethod.RelatedMethods[index].ServiceName);
            Assert.AreEqual(1, serviceMethod.RelatedMethods[index].ParameterMapping.Count);
            Assert.AreEqual("entity", serviceMethod.RelatedMethods[index].ParameterMapping[0].Name);
            Assert.AreEqual("Course", serviceMethod.RelatedMethods[index].ParameterMapping[0].DotNetType);
            Assert.AreEqual("dl", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeNamespace);
            Assert.AreEqual("dl.Course", serviceMethod.RelatedMethods[index].ParameterMapping[0].RelativeName);
            Assert.AreEqual(RelationshipTypeEnum.ManyToMany, serviceMethod.RelatedMethods[index].RelationshipType);

            #endregion
        }

        #endregion

        #region .   Exposed Entity - Store.Multiple.ManyToMany



        #endregion

        #region .   Exposed Entity - Select.Single

        [Test]
        public void ExposedEntity_Student_SelectSingle()
        {
            ServiceMethod serviceMethod = _businessLayer.BusinessServices["StudentService"].Methods["StudentSearch"];

            Assert.AreEqual(false, serviceMethod.IsCustomImplementation);
            Assert.AreEqual("StudentSearch", serviceMethod.Name);
            Assert.AreEqual("dl.Student", serviceMethod.ReturnType.FullName);
            Assert.AreEqual(true, serviceMethod.ReturnType.IsEnumerable);
            Assert.AreEqual(true, serviceMethod.ReturnType.IsKnownType);
            Assert.AreEqual("dl", serviceMethod.ReturnType.Namespace);
            Assert.AreEqual("Student", serviceMethod.ReturnType.Type);
            Assert.AreEqual(ServiceFeatureEnum.RetrieveMultiple, serviceMethod.ServiceFeature);
            Assert.AreEqual("gsp_StudentSearch", serviceMethod.StoredProcedureName);

            #region Parameters

            // Now Process the Parameters
            Assert.AreEqual(5, serviceMethod.Parameters.Count);

            int paramIndex = 0;
            Assert.AreEqual("whereClause", serviceMethod.Parameters[paramIndex].Name);
            Assert.AreEqual("string", serviceMethod.Parameters[paramIndex].DotNetType);
            Assert.AreEqual("", serviceMethod.Parameters[paramIndex].RelativeNamespace);
            Assert.AreEqual("string", serviceMethod.Parameters[paramIndex].RelativeName);

            paramIndex ++;
            Assert.AreEqual("sortExpression", serviceMethod.Parameters[paramIndex].Name);
            Assert.AreEqual("string", serviceMethod.Parameters[paramIndex].DotNetType);
            Assert.AreEqual("", serviceMethod.Parameters[paramIndex].RelativeNamespace);
            Assert.AreEqual("string", serviceMethod.Parameters[paramIndex].RelativeName);

            paramIndex++;
            Assert.AreEqual("startingRecord", serviceMethod.Parameters[paramIndex].Name);
            Assert.AreEqual("int", serviceMethod.Parameters[paramIndex].DotNetType);
            Assert.AreEqual("", serviceMethod.Parameters[paramIndex].RelativeNamespace);
            Assert.AreEqual("int", serviceMethod.Parameters[paramIndex].RelativeName);

            paramIndex++;
            Assert.AreEqual("pageSize", serviceMethod.Parameters[paramIndex].Name);
            Assert.AreEqual("int", serviceMethod.Parameters[paramIndex].DotNetType);
            Assert.AreEqual("", serviceMethod.Parameters[paramIndex].RelativeNamespace);
            Assert.AreEqual("int", serviceMethod.Parameters[paramIndex].RelativeName);

            paramIndex++;
            Assert.AreEqual("whereClauseParameters", serviceMethod.Parameters[paramIndex].Name);
            Assert.AreEqual("params string[]", serviceMethod.Parameters[paramIndex].DotNetType);
            Assert.AreEqual("", serviceMethod.Parameters[paramIndex].RelativeNamespace);
            Assert.AreEqual("params string[]", serviceMethod.Parameters[paramIndex].RelativeName);

            #endregion

            #region Parameter Mappings

            // Now Process Parameter mappings
            Assert.AreEqual(0, serviceMethod.StoredProcedureMapping.Count);

            #endregion
        }

        #endregion

    }
}