#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.DomainModeler;
using dnp.ServiceModel.Configuration;
using dnp.Framework.Scripting;

namespace dnp.ServiceModel
{
    public class BusinessLayer
    {
        private BusinessServiceCollection _businessServices = new BusinessServiceCollection();
        BusinessLayerFeatures _features;
        private dnpModel _model;
        private MetaData _metaData;

        public BusinessLayer(dnpModel model)
        {
            _model = model;
            _features = new BusinessLayerFeatures(model.Definition.BusinessLayerDefinition.Features, this.Model);
            _metaData = new MetaData(model, model.Definition.BusinessLayerDefinition.MetaData);
            // Create a business service for each defined.
            foreach (BusinessServiceDefinition serviceDef in this.Model.Definition.BusinessLayerDefinition.BusinessServiceDefinitions)
                this.BusinessServices.Add(new BusinessService(this, serviceDef));

            // This must be done as a second pass as relationships require information about all service definitions.
            foreach (BusinessService bs in this.BusinessServices)
                bs.ProcessRelationships();
        }

        /// <summary>
        /// This namespace defines the root or relative root for the services layer. This namespace
        /// is appened to the root namespace to provide a full namespace for the service. Each service
        /// definition can also add its own relative namespace too.
        /// </summary>
        /// <example>
        /// Business.Services
        /// </example>
        public string RelativeServiceLayerNamespace
        {
            get { return Naming.EvaluateNamingExpression(this.Model.Definition.BusinessLayerDefinition.RelativeServiceLayerNamespace, this); }
        }
        public string ServiceLayerUnitTestNamespace
        {
            get { return (this.Model.RootNamespace.Length == 0 ? "" : this.Model.RootNamespace + ".") + this.RelativeServiceLayerUnitTestNamespace; }
        }

        public string ServiceLayerNamespace
        {
            get { return (this.Model.RootNamespace.Length == 0 ? "" : this.Model.RootNamespace + ".") + this.RelativeServiceLayerNamespace; }
        }

        /// <summary>
        /// This namespace defines the root or relative root for the domain layer. This namespace
        /// is appened to the root namespace to provide a full namespace for the domain.
        /// </summary>
        /// <example>
        /// Business.Entities
        /// </example>
        public string RelativeDomainLayerNamespace
        {
            get { return Naming.EvaluateNamingExpression(this.Model.Definition.BusinessLayerDefinition.RelativeDomainLayerNamespace, this); }
        }
        public string RelativeServiceLayerUnitTestNamespace
        {
            get { return Naming.EvaluateNamingExpression(this.Model.Definition.BusinessLayerDefinition.RelativeServiceLayerNamespace, this); }
        }
        public string DomainLayerNamespace
        {
            get { return DomainModeler.Model.FixNamespace(this.Model.RootNamespace + "." + this.RelativeDomainLayerNamespace); }
        }
        public BusinessServiceCollection BusinessServices
        {
            get { return _businessServices; }
            set { _businessServices = value; }
        }

        public string FullEntityName(Entity entity)
        {
            return DomainModeler.Model.FixNamespace(this.FullEntityNamespace(entity) + "." + entity.Name);
        }
        public string FullEntityNamespace(Entity entity)
        {
            return DomainModeler.Model.FixNamespace(this.DomainLayerNamespace +  "."  + entity.RelativeNamespace);
        }
        public dnpModel Model
        {
            get { return _model; }
        }

        public Domain Domain
        {
            get { return this.Model.Domain; }
        }

        public MetaData MetaData
        {
            get { return _metaData; }
        }

        public BusinessLayerFeatures Features
        {
            get { return _features; }
            set { _features = value; }
        }
    }
}