#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion

using System;

namespace dnp.ServiceModel
{
    public class ParameterMap
    {
        private string _parameterName = "";
        private string _propertyMapping = "";
        private bool _isOutput;
        private Type _dotNetType;
        private bool _isNullable;
        private string _nullablePropertyParameterCheck = "";
        private int _dbTypeSize;

        public ParameterMap()
        {
        }


        public ParameterMap(string parameterName, string propertyMapping, Type dotNetType, bool isOutput, bool isNullable, string nullablePropertyParamterCheck, int dbTypeSize)
        {
            _parameterName = parameterName;
            _propertyMapping = propertyMapping;
            _isOutput = isOutput;
            _dotNetType = dotNetType;
            _isNullable = isNullable;
            _dbTypeSize = dbTypeSize;
            _nullablePropertyParameterCheck = nullablePropertyParamterCheck;
        }

        /// <summary>
        /// Defined stored procedure parameter to map
        /// </summary>
        public string ParameterName
        {
            get { return _parameterName; }
            set { _parameterName = value; }
        }

        /// <summary>
        /// The property (including sub property) to map to the stored procedure parameter
        /// </summary>
        /// <example>
        /// request.CustomerName
        /// customerName
        /// </example>
        public string PropertyMapping
        {
            get { return _propertyMapping; }
            set { _propertyMapping = value; }
        }

        /// <summary>
        /// Determine is the parameter can be updated ie identity or concurrency values
        /// </summary>
        public bool IsOutput
        {
            get { return _isOutput; }
            set { _isOutput = value; }
        }

        public Type DotNetType
        {
            get { return _dotNetType; }
            set { _dotNetType = value; }
        }

        /// <summary>
        /// The DbType of this property
        /// </summary>
        public string DbType
        {
            get
            {
                string dbType = this.DotNetType.Name;
                // Need to do some conversions to make it a correct DbType
                dbType = dbType.Replace("[]", "").Replace("XmlDocument", "Xml");

                return dbType;
            }
        }

        /// <summary>
        /// Some values are optional, this property determines if the property can be nullable.
        /// </summary>
        public bool IsNullable
        {
            get { return _isNullable; }
            set { _isNullable = value; }
        }

        /// <summary>
        /// This property is used to determine how to test for a null value. In many cases it will be the same field as the property mapping field. However, if the property mapping field
        /// is on a child object then this property is used to specify the parent property (which also maybe null) to avoid accessing a null parents child properties.
        /// </summary>
        public string NullablePropertyParameterCheck
        {
            get { return _nullablePropertyParameterCheck; }
            set { _nullablePropertyParameterCheck = value; }
        }

        /// <summary>
        /// This property is used by out parameters. Its a required field of the Microsoft Data Access Blocks AddOutParameter.
        /// </summary>
        public int DbTypeSize
        {
            get { return _dbTypeSize; }
            set { _dbTypeSize = value; }
        }
    }
}
