#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using dnp.DomainModeler;

namespace dnp.ServiceModel
{
    public class ServiceMethod
    {
        private readonly ServiceFeatureEnum _serviceFeature;
        private readonly string _name;
        private readonly string _storedProcedureName;
        private readonly bool _isCustomImplementation;

        private readonly BusinessLayer _businessLayer;
        private readonly ParameterMapCollection _storedProcedureMapping = new ParameterMapCollection();
        private ServiceReturnType _returnType = new ServiceReturnType();
        private readonly ServiceMethodRelationshipCollection _serviceMethodRelationships = new ServiceMethodRelationshipCollection();
        private readonly ServiceMethodParameterCollection _parameters = new ServiceMethodParameterCollection();
        private readonly Entity _sourceEntity;
        private ServiceMethodTypeEnum _serviceMethodType;
        private ServiceReturnTypePropertyMapCollection _returnTypePropertyMapping = new ServiceReturnTypePropertyMapCollection();
        private BusinessService _parent;

        public ServiceMethod(BusinessService parent, string stordProcedureName, ServiceFeatureEnum serviceFeature, string name, ServiceMethodTypeEnum serviceMethodType,
                             bool isCustomImplementation, ServiceReturnType returnType, Entity sourceEntity)
        {
            _parent = parent;
            _businessLayer = parent.Parent;
            _storedProcedureName = stordProcedureName;
            _serviceFeature = serviceFeature;
            _name = name;
            _isCustomImplementation = isCustomImplementation;
            _returnType = returnType;
            _sourceEntity = sourceEntity;
            _serviceMethodType = serviceMethodType;
        }

        public BusinessLayer BusinessLayer
        {
            get { return _businessLayer; }
        }

        public BusinessService Parent
        {
            get{ return _parent;}
        }

        public ServiceFeatureEnum ServiceFeature
        {
            get { return _serviceFeature; }
        }

        public ServiceMethodTypeEnum ServiceMethodType
        {
            get { return _serviceMethodType; }
        }

        public string Name
        {
            get { return _name; }
        }

        public string StoredProcedureName
        {
            get { return _storedProcedureName; }
        }

        /// <summary>
        /// Determines if the defined stored procedure is used or if one will be calculated. If calculated it
        /// informs the scripts to also auto-generate the stored procedure to satisfy this method. This property
        /// should be set to true if a custom stored procedure is being used to statify the implemenation of the 
        /// service method.
        /// </summary>
        public bool IsCustomImplementation
        {
            get { return _isCustomImplementation; }
        }

        public ParameterMapCollection StoredProcedureMapping
        {
            get { return _storedProcedureMapping; }
        }

        public ServiceReturnType ReturnType
        {
            get { return _returnType; }
            internal set { _returnType = value; }
        }

        public ServiceMethodRelationshipCollection RelatedMethods
        {
            get { return _serviceMethodRelationships; }
        }

        /// <summary>
        /// Collection of paramters which make up the service method signature
        /// </summary>
        public ServiceMethodParameterCollection Parameters
        {
            get { return _parameters; }
        }

        /// <summary>
        /// The entity this method is based on. This maybe different to the return type, and may also be null if the method is not based
        /// on an defined entity.
        /// </summary>
        public Entity SourceEntity
        {
            get { return _sourceEntity; }
        }

        public ServiceReturnTypePropertyMapCollection ReturnTypePropertyMapping
        {
            get { return _returnTypePropertyMapping; }
        }

        public override string ToString()
        {
            return this.Name;
        }
    }
}