#region License
// (c) Garry McGlennon 2009
// 
// This project is made available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// 
// View the project and the latest code at http://dnpframework.codeplex.com/
// 
// All other rights reserved.
// 
// blog: www.dotNetProfessional.com/blog/
#endregion
using System;
using dnp.DomainModeler;

namespace dnp.ServiceModel
{
    /// <summary>
    /// This class represents the return type of a service method.
    /// </summary>
    public class ServiceReturnType
    {
        private readonly string _type = "";
        private readonly string _namespace = "";
        private readonly bool _isEnumerable;
        private readonly bool _isKnownType;
        private readonly dnpModel _model;

        public ServiceReturnType()
        {
        }

        public ServiceReturnType(dnpModel model, bool isEnumerable, bool isKnownType, string @namespace, string type)
        {
            _model = model;
            _isEnumerable = isEnumerable;
            _isKnownType = isKnownType;
            _namespace = @namespace;
            _type = type;
        }

        public string Type
        {
            get { return _type; }
        }

        public string Namespace
        {
            get { return _namespace; }
        }

        public string FullName
        {
            get { return DomainModeler.Model.FixNamespace(this.Namespace + "." + this.Type); }
        }

        public bool IsEnumerable
        {
            get { return _isEnumerable; }
        }

        public bool IsKnownType
        {
            get { return _isKnownType; }
        }

        public dnpModel Model
        {
            get { return _model; }
        }

        public Entity KnownType
        {
            get
            {
                if (this.IsKnownType)
                    return this.Model.Domain.Entities[this.Type];


                throw new ArgumentException("Type is defined as unknown.");
            }
        }
    }
}
